from dataclasses import dataclass, asdict
from datetime import datetime

import pandas as pd

@dataclass
class MarketNewsData:
    date: datetime
    title: str
    source: str
    url: str
    section: str

    def as_dataframe(self) -> pd.DataFrame:
        """Converts the object to a pandas DataFrame."""
        return pd.DataFrame([asdict(self)])

    def insert_into_pg(self, conn):
        cursor = conn.cursor()
        insert_query = """INSERT INTO market_news (date, title, source, url, section) VALUES (%s, %s, %s, %s, %s);"""
        cursor.execute(insert_query, (self.date, self.title, self.source, self.url, self.section))
        conn.commit()



import pandas as pd

class SpyData:
    def __init__(self, data):
        self.quoteDate = [i.get("quoteDate", None) for i in data]
        self.pOpen = [i.get("pOpen", None) for i in data]
        self.high = [i.get("high", None) for i in data]
        self.low = [i.get("low", None) for i in data]
        self.pClose = [i.get("pClose", None) for i in data]
        self.volume = [i.get("volume", None) for i in data]
        self.advance = [i.get("advance", None) for i in data]
        self.decline = [i.get("decline", None) for i in data]
        self.upVol = [i.get("upVol", None) for i in data]
        self.downVol = [i.get("downVol", None) for i in data]
        self.upMoney = [i.get("upMoney", None) for i in data]
        self.downMoney = [i.get("downMoney", None) for i in data]
        self.rollingEPS = [i.get("rollingEPS", None) for i in data]
        self.sentiment = [i.get("sentiment", None) for i in data]
        self.na = [i.get("na", None) for i in data]
        self.ai = [i.get("ai", None) for i in data]
        self.netSentRatio = [i.get("netSentRatio", None) for i in data]
        self.netBuyRatio = [i.get("netBuyRatio", None) for i in data]
        self.peakTroughShorts = [i.get("peakTroughShorts", None) for i in data]
        self.avg20Day = [i.get("avg20Day", None) for i in data]
        self.targetRatio = [i.get("targetRatio", None) for i in data]


        # Construct self.data_dict
        self.data_dict = {
            "quote_date": self.quoteDate,
            "pre_open": self.pOpen,
            "high": self.high,
            "low": self.low,
            "pre_lose": self.pClose,
            "volume": self.volume,
            "advance": self.advance,
            "decline": self.decline,
            "up_vol": self.upVol,
            "down_vol": self.downVol,
            "up_money": self.upMoney,
            "down_money": self.downMoney,
            "rolling_eps": self.rollingEPS,
            "sentiment": self.sentiment,
            "na": self.na,
            "ai": self.ai,
            "net_sentiment_ratio": self.netSentRatio,
            "net_buy_ratio": self.netBuyRatio,
            "peak_trough_shorts": self.peakTroughShorts,
            "avg_20_day": self.avg20Day,
            "target_ratio": self.targetRatio
        }


        self.as_dataframe = pd.DataFrame(self.data_dict)

class TopSentimentHeatmap:
    def __init__(self, data):
        self.ticker = [i.get('ticker', None) for i in data]
        self.total = [i.get('total', None) for i in data]
        self.avg_sentiment = [i.get('avgSent', None) for i in data]
        self.company = [i.get('company', None) for i in data]
        self.next_eps_date = [i.get('nextEPSDate', None) for i in data]
        self.release_time = [i.get('releaseTime', None) for i in data]
        self.confirm_date = [i.get('confirmDate', None) for i in data]
        self.sentiment = [i.get('sentiment', None) for i in data]


        self.data_dict = { 

            'ticker': self.ticker,
            'total': self.total,
            'avg_sentiment': self.avg_sentiment,
            'company': self.company,
            'next_er': self.next_eps_date,
            'release_time': self.release_time,
            'confirmation_date': self.confirm_date,
            'sentiment': self.sentiment
        }


        self.as_dataframe = pd.DataFrame(self.data_dict)



class UpcomingRussellAndSectors:
    def __init__(self, data):
        self.earningsDate = [i.get("earningsDate", None) for i in data]
        self.epsGrowth = [i.get("epsGrowth", None) for i in data]
        self.maxDate = [i.get("maxDate", None) for i in data]
        self.revGrowth = [i.get("revGrowth", None) for i in data]
        self.secID = [i.get("secID", None) for i in data]
        self.sector = [i.get("sector", None) for i in data]
        self.sectorName = [i.get("sectorName", None) for i in data]
        self.surprise = [i.get("surprise", None) for i in data]
        self.total = [i.get("total", None) for i in data]
        self.week = [i.get("week", None) for i in data]

        # Construct self.data_dict
        self.data_dict = {
            "earnings_date": self.earningsDate,
            "eps_growth": self.epsGrowth,
            "max_date": self.maxDate,
            "revenue_growth": self.revGrowth,
            "sector_id": self.secID,
            "sector": self.sector,
            "sector_name": self.sectorName,
            "surprise": self.surprise,
            "total": self.total,
            "week": self.week
        }



        self.as_dataframe = pd.DataFrame(self.data_dict)




class DatedChartData:
    def __init__(self, data):
        self.quoteDate = [i.get("quoteDate", None) for i in data]
        self.ticker = [i.get("ticker", None) for i in data]
        self.name = [i.get("name", None) for i in data]
        self.pClose = [i.get("pClose", None) for i in data]
        self.pOpen = [i.get("pOpen", None) for i in data]
        self.high = [i.get("high", None) for i in data]
        self.low = [i.get("low", None) for i in data]
        self.volume = [i.get("volume", None) for i in data]
        self.fwdEPS = [i.get("fwdEPS", None) for i in data]
        self.arrow = [i.get("arrow", None) for i in data]
        self.shortShares = [i.get("shortShares", None) for i in data]
        self.sentiment = [i.get("sentiment", None) for i in data]
        self.ewUserNo = [i.get("ewUserNo", None) for i in data]
        self.traderStatus = [i.get("traderStatus", None) for i in data]
        self.investorStatus = [i.get("investorStatus", None) for i in data]
        self.epsOpenDate = [i.get("epsOpenDate", None) for i in data]
        self.avwap = [i.get("avwap", None) for i in data]

        # Construct self.data_dict
        self.data_dict = {
            "quote_date": self.quoteDate,
            "ticker": self.ticker,
            "name": self.name,
            "pre_close": self.pClose,
            "pre_open": self.pOpen,
            "high": self.high,
            "low": self.low,
            "volume": self.volume,
            "forward_eps": self.fwdEPS,
            "arrow": self.arrow,
            "short_shares": self.shortShares,
            "sentiment": self.sentiment,
            "ew_user_no": self.ewUserNo,
            "trader_status": self.traderStatus,
            "investor_status": self.investorStatus,
            "eps_open_date": self.epsOpenDate,
            "aggregate_vwap": self.avwap
        }


        self.as_dataframe = pd.DataFrame(self.data_dict)



class Messages:
    def __init__(self, data):

        self.artDate = [i.get("artDate", None) for i in data]
        self.artNo = [i.get("artNo", None) for i in data]
        self.artType = [i.get("artType", None) for i in data]
        self.imageName = [i.get("imageName", None) for i in data]
        self.summary = [i.get("summary", None) for i in data]
        self.tickers = [i.get("tickers", None) for i in data]
        self.title = [i.get("title", None) for i in data]

        # Construct self.data_dict
        self.data_dict = {
            "art_date": self.artDate,
            "art_no": self.artNo,
            "art_type": self.artType,
            "image_name": self.imageName,
            "summary": self.summary,
            "tickers": self.tickers,
            "title": self.title
        }


        self.as_dataframe = pd.DataFrame(self.data_dict)



class Pivots:
    def __init__(self, data):
        self.ticker = [i.get("ticker", None) for i in data]
        self.company = [i.get("company", None) for i in data]
        self.listType = [i.get("listType", None) for i in data]
        self.lastTrade = [i.get("lastTrade", None) for i in data]
        self.pivot_point = [i.get("pp", None) for i in data]
        self.resistance_1 = [i.get("r1", None) for i in data]
        self.support_1 = [i.get("s1", None) for i in data]
        self.resistance_2 = [i.get("r2", None) for i in data]
        self.support_2 = [i.get("s2", None) for i in data]
        self.resistance_3 = [i.get("r3", None) for i in data]
        self.support_3 = [i.get("s3", None) for i in data]

        # Construct self.data_dict
        self.data_dict = {
            "ticker": self.ticker,
            "company": self.company,
            "list_type": self.listType,
            "last_trade": self.lastTrade,
            "pivot_point": self.pivot_point,
            "resistance_1": self.resistance_1,
            "support_1": self.support_1,
            "resistance_2": self.resistance_2,
            "support_2": self.support_2,
            "resistance_3": self.resistance_3,
            "support_3": self.support_3
        }


        self.as_dataframe = pd.DataFrame(self.data_dict)




class TodaysResults:
    def __init__(self, data):
        self.earningsGrowth = [i.get("earningsGrowth", None) for i in data]
        self.earningsSurprise = [i.get("earningsSurprise", None) for i in data]
        self.eps = [i.get("eps", None) for i in data]
        self.epsDate = [i.get("epsDate", None) for i in data]
        self.estimate = [i.get("estimate", None) for i in data]
        self.ewGrade = [i.get("ewGrade", None) for i in data]
        self.fileName = [i.get("fileName", None) for i in data]
        self.highEstimate = [i.get("highEstimate", None) for i in data]
        self.lowEstimate = [i.get("lowEstimate", None) for i in data]
        self.name = [i.get("name", None) for i in data]
        self.prevEarningsGrowth = [i.get("prevEarningsGrowth", None) for i in data]
        self.prevRevenueGrowth = [i.get("prevRevenueGrowth", None) for i in data]
        self.pwrRating = [i.get("pwrRating", None) for i in data]
        self.quarter = [i.get("quarter", None) for i in data]
        self.revenue = [i.get("revenue", None) for i in data]
        self.revenueEstimate = [i.get("revenueEstimate", None) for i in data]
        self.revenueGrowth = [i.get("revenueGrowth", None) for i in data]
        self.revenueSurprise = [i.get("revenueSurprise", None) for i in data]
        self.subject = [i.get("subject", None) for i in data]
        self.summary = [i.get("summary", None) for i in data]
        self.ticker = [i.get("ticker", None) for i in data]
        self.whisper = [i.get("whisper", None) for i in data]

        # Construct self.data_dict
        self.data_dict = {
            "earnings_growth": self.earningsGrowth,
            "earnings_surprise": self.earningsSurprise,
            "eps": self.eps,
            "er_date": self.epsDate,
            "estimate": self.estimate,
            "ew_grade": self.ewGrade,
            "file_name": self.fileName,
            "high_estimate": self.highEstimate,
            "low_estimate": self.lowEstimate,
            "name": self.name,
            "prev_earnings_growth": self.prevEarningsGrowth,
            "prev_revenue_growth": self.prevRevenueGrowth,
            "pwrRating": self.pwrRating,
            "quarter": self.quarter,
            "revenue": self.revenue,
            "revenue_estimate": self.revenueEstimate,
            "revenue_growth": self.revenueGrowth,
            "revenue_surprise": self.revenueSurprise,
            "subject": self.subject,
            "summary": self.summary,
            "ticker": self.ticker,
            "whisper": self.whisper
        }



        self.as_dataframe = pd.DataFrame(self.data_dict)



class CalData:
    def __init__(self, data):

        self.company = [i.get("company", None) for i in data]
        self.confirmDate = [i.get("confirmDate", None) for i in data]
        self.epsTime = [i.get("epsTime", None) for i in data]
        self.nextEPSDate = [i.get("nextEPSDate", None) for i in data]
        self.q1EstEPS = [i.get("q1EstEPS", None) for i in data]
        self.q1RevEst = [i.get("q1RevEst", None) for i in data]
        self.qDate = [i.get("qDate", None) for i in data]
        self.qSales = [i.get("qSales", None) for i in data]
        self.quarterDate = [i.get("quarterDate", None) for i in data]
        self.releaseTime = [i.get("releaseTime", None) for i in data]
        self.ticker = [i.get("ticker", None) for i in data]
        self.total = [i.get("total", None) for i in data]

        # Construct self.data_dict
        self.data_dict = {
            "company": self.company,
            "confirmation_date": self.confirmDate,
            "earnings_time": self.epsTime,
            "next_er_date": self.nextEPSDate,
            "q1_estimate_eps": self.q1EstEPS,
            "q1_revenue_est": self.q1RevEst,
            "q_date": self.qDate,
            "q_sales": self.qSales,
            "quarter_date": self.quarterDate,
            "release_time": self.releaseTime,
            "ticker": self.ticker,
            "total": self.total
        }



        self.as_dataframe = pd.DataFrame(self.data_dict)
    


import sys
from pathlib import Path
import os
# Add the project directory to the sys.path
project_dir = str(Path(__file__).resolve().parents[2])
if project_dir not in sys.path:
    sys.path.append(project_dir)
import re
from _markets.list_sets.ticker_lists import most_active_tickers
from .rss_models import Feed,Entries
import feedparser
from ..polygonio.async_polygon_sdk import Polygon
import pandas as pd
import requests
import asyncio
import aiohttp
import xml.etree.ElementTree as ET
from .rss_helpers import fix_xml_attributes, xml_to_dataframe,extract_xml_data


from bs4 import BeautifulSoup



most_active_tickers = set(most_active_tickers)

polygon=Polygon()


from datetime import datetime, timedelta

class RSSSDK:
    def __init__(self):
        self.sec_base = "https://www.sec.gov"

        self.headers = {
            "Accept": "application/json",
            "User-Agent":"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/115.0.0.0 Safari/537.36"
            }
        self.today = datetime.now().strftime('%Y-%m-%d')
        self.yesterday = (datetime.now() - timedelta(days=1)).strftime('%Y-%m-%d')
        self.tomorrow = (datetime.now() + timedelta(days=1)).strftime('%Y-%m-%d')
        self.thirty_days_ago = (datetime.now() - timedelta(days=30)).strftime('%Y-%m-%d')
        self.thirty_days_from_now = (datetime.now() + timedelta(days=30)).strftime('%Y-%m-%d')
        self.fifteen_days_ago = (datetime.now() - timedelta(days=15)).strftime('%Y-%m-%d')
        self.fifteen_days_from_now = (datetime.now() + timedelta(days=15)).strftime('%Y-%m-%d')
        self.eight_days_from_now = (datetime.now() + timedelta(days=8)).strftime('%Y-%m-%d')
        self.eight_days_ago = (datetime.now() - timedelta(days=8)).strftime('%Y-%m-%d')

    

    async def fetch_new_posts(self,url):
        last_update_time = None

        while True:
            async with aiohttp.ClientSession(headers=self.headers) as session:
                async with session.get(url) as resp:
         

                    feed_data = await resp.json()

                    for post in feed_data:
                        post_time = post['date']
                        if post_time != last_update_time:
                            last_update_time = post_time
                            return post

            await asyncio.sleep(60)  # Wait for 60 seconds before fetching the feed again



    async def company_filings(self, ticker, as_dataframe:bool=False, limit:str='100'):
        """
        Takes in a ticker - converts to CIK - passes to SEC Rss feed
        
        Arguments:

        >>> as_dataframe: optional - returns a dataframe of company filings


        >>> limit: optional - the number of filings to search (default 100)

        """

        cik = await polygon.company_info(ticker)
        if cik is not None:
            cik = cik.cik
            rss_url = f"https://data.sec.gov/rss?cik={cik}&count={limit}"
            print(rss_url)

            # Parse the RSS feed
            feed = feedparser.parse(rss_url, request_headers=self.headers)

            data = Feed(feed)
            
            entries = data.entries
            

            entries = Entries(entries)
        
            if as_dataframe == True:
                
                entries.as_dataframe.to_csv('data/rss_feeds/commpany_filings.csv')
                return entries.as_dataframe
            # Filter the links based on the dates
            links_today = [link for link, date in zip(entries.links, entries.filing_dates) if date == self.today]

        
            return links_today

    
    async def scan_filing_type(self, ticker:str, type:str='4'):
        """
        Returns SEC index.htm for a specific filing type.

        Arguments:


        >>> ticker: REQUIRED - the ticker to query

        >>> type: optional - the type of filing to query (default 4)

            4 - form 4 
            10-Q 
            10-K
        """
        cik = await polygon.company_info(ticker)
        if cik is not None:
            cik = cik.cik
            rss_url = f"https://data.sec.gov/rss?cik={cik}&count=100&type={type}"
    
            print(rss_url)

            # Parse the RSS feed
            feed = feedparser.parse(rss_url, request_headers=self.headers)

            data = Feed(feed)
            
            entries = data.entries
            

            entries = Entries(entries)
    
            # Filter the links based on the dates
            links_today = [link for link, date in zip(entries.links, entries.filing_dates) if date >= self.today]

        
            return links_today
    async def get_filing_urls(self, ticker:str):
        try:
            filings = await self.company_filings(ticker)
        except aiohttp.client.ClientConnectorError:
            print(f'Error - Ticker')

        if filings is not None:
       
            for filing in filings:
                response = requests.get(filing, headers=self.headers)

                if response.status_code == 200:
                    soup = BeautifulSoup(response.text, 'html.parser')
                    table = soup.find('table', class_='tableFile')
                    
                    if table:
                        for row in table.find_all('tr')[1:]:  # Skip the header row
                            cells = row.find_all('td')
                            description = cells[1].text.strip()
                            document_link = cells[2].a['href'].strip() if cells[2].a else ""


                            #form 4 
                            if "xslF" not in document_link and document_link.endswith('.xml') and 'xsl144' not in document_link and 'primary_doc' not in document_link:
                                # Process the document link
                                print(description, document_link)
                                
                                xml_url = self.sec_base + document_link
                                response = requests.get(xml_url, headers=self.headers)
                                
                                if response.status_code == 200:
                                    xml_content = response.text if not response.text.startswith('<!DOCTYPE html') else None
                                    if xml_content is not None:
                                        
                                        file_name = 'company_filing_form4.xml'  # Get the last part of the URL as the file name
                                        
                                        with open(file_name, 'w', encoding='utf-8') as file:
                                            file.write(xml_content)
                                        print(f"XML file saved as {file_name}")
                                        # Parse the XML file
                                        tree = ET.parse(file_name)
                                        root = tree.getroot()
                                        
                                        df = xml_to_dataframe(file_name)
                                        df['ticker'] = ticker
                                        data_dict = df.to_dict(orient='dict')
                                        for key, value in data_dict.items():
                                            data_dict[key] = value[0]


                                        df = pd.DataFrame(data_dict, index=df['ticker'])

                                        print(df.transpose())
                                       
                                                                                    
                            if "primary_doc" in document_link and 'xslF' not in document_link and document_link.endswith('.xml'):
                                # Process the document link for xsl144 filings
                                print(description, document_link)
                                
                                xml_url = self.sec_base + document_link
                                response = requests.get(xml_url, headers=self.headers)
                                
                                if response.status_code == 200:
                                    xml_content = response.text if not response.text.startswith('<!DOCTYPE html') else None
                                    if xml_content is not None:
                                        
                                        file_name = 'company_filing_144.xml'  # Define a specific name for xsl144 filings
                                        
                                        with open(file_name, 'w', encoding='utf-8') as file:
                                            file.write(xml_content)
                                        print(f"XML file saved as {file_name}")
                                        
                                        # Parse the XML file
                                        tree = ET.parse(file_name)
                                        root = tree.getroot()
                                        
                                        df = xml_to_dataframe(file_name)
                                        df['ticker'] = ticker
                                        data_dict = df.to_dict(orient='dict')
                                        for key, value in data_dict.items():
                                            data_dict[key] = value[0]
                                        
                                        df = pd.DataFrame(data_dict, index=[ticker])
                                        # Remove namespaces from column names
                                        df.columns = [col.split('}')[-1] if '}' in col else col for col in df.columns]

                                

                                        # Print the cleaned DataFrame
                                        print(df.transpose())          
                                        return df                          

                        else:
                            if document_link.endswith('xml'):
                                print('https://www.sec.gob/'+document_link)
                            






import disnake
from disnake.ext import commands
import pandas as pd
from apis.polygonio.polygon_options import PolygonOptions
from tabulate import tabulate
opts = PolygonOptions(user='postgres', database='fudstop')


class SPXCOG(commands.Cog):
    def __init__(self, bot):
        self.bot=bot


    @commands.slash_command()
    async def spx(self, inter):
        pass




    @spx.sub_command()
    async def monitor_change_percent(self, inter:disnake.AppCmdInter):
        await inter.response.defer()
        counter = 0
        while True:
            counter = counter + 1

            fetched_price = await opts.get_price(ticker='SPX')
            lower_price = float(fetched_price) - 15
            upper_price = float(fetched_price) + 15

            # Fetch options
            options = await opts.get_option_chain_all(
                underlying_asset='I:SPX', 
                strike_price_gte=lower_price, 
                strike_price_lte=upper_price, 
                expiration_date='2024-01-03'
            )


            iv = [round(vol * 100, 2) for vol in options.implied_volatility]
            # Your options data
            cr = options.change_percent
            cp = options.contract_type
            ticker = options.underlying_ticker
            price = options.underlying_price
            theta = [round(option, 2) for option in options.theta]
            delta = [round(option, 2) for option in options.delta]
            gamma = [round(option, 2) for option in options.gamma]
            vol = options.volume
            strike = options.strike

            # Initialize dictionaries for calls and puts
            calls_data = {'strike': [], 'iv': [], 'change%': [], 'theta': [], 'vol': [],}
            puts_data = {'strike': [], 'iv': [], 'change%': [], 'theta': [], 'vol': []}

            # Separate out calls and puts
            for i, contract_type in enumerate(cp):
                if contract_type.lower() == 'call':
                    calls_data['strike'].append(strike[i])
                    calls_data['iv'].append(iv[i])
                    calls_data['change%'].append(cr[i])
                    calls_data['theta'].append(theta[i])
                    calls_data['vol'].append(vol[i])
                    
                elif contract_type.lower() == 'put':
                    puts_data['strike'].append(strike[i])
                    puts_data['iv'].append(iv[i])
                    puts_data['change%'].append(cr[i])
                    puts_data['theta'].append(theta[i])
                    puts_data['vol'].append(vol[i])
                    

            # Convert to DataFrames if needed
            calls_df = pd.DataFrame(calls_data)
            puts_df = pd.DataFrame(puts_data)


            # Get the new price from a separate function

            # Determine the length of the other columns


            calls_table = tabulate(calls_df, headers='keys', tablefmt='fancy', showindex=False)
            puts_table = tabulate(puts_df, headers='keys', tablefmt='fancy', showindex=False)

        
            color = disnake.Colour.dark_red()

            embed = disnake.Embed(title=f"SPX Change % Monitor - | 0DTE", description=f"```py\n${fetched_price}```", color=color)
            embed.add_field(name=f"# > CALLS:", value=f"```py\n{calls_table}```", inline=False)
            embed.add_field(name=f"_ _ _ _", value=f"# > SPX: **${fetched_price}**", inline=False)
            embed.add_field(name="# > PUTS:", value=f"```py\n{puts_table}```", inline=False)
            await inter.edit_original_message(embed=embed)

            if counter == 150:
                break

def setup(bot: commands.Bot):
    bot.add_cog(SPXCOG(bot))


import sys
from pathlib import Path

# Add the project directory to the sys.path
project_dir = str(Path(__file__).resolve().parents[1])
if project_dir not in sys.path:
    sys.path.append(project_dir)

import pandas as pd
from datetime import datetime, timedelta
from apis.polygonio.mapping import option_condition_desc_dict,OPTIONS_EXCHANGES
indices_list = ["SPX", "SPXW", "NDX", "VIX", "VVIX"]



class UniversalSnapshot:
    def __init__(self, results):
        self.today = datetime.now().strftime('%Y-%m-%d')
        self.yesterday = (datetime.now() - timedelta(days=1)).strftime('%Y-%m-%d')
        self.tomorrow = (datetime.now() + timedelta(days=1)).strftime('%Y-%m-%d')

        self.break_even_price = [i.get('session.break_even_price') for i in results]
        self.change = [i.get('session.change', None) for i in results]
        self.change_percent = [i.get('session.change_percent') for i in results]
        self.early_trading_change = [i.get('session.early_trading_change') for i in results]
        self.early_trading_change_percent = [i.get('session.early_trading_change_percent') for i in results]
        self.close = [i.get('session.close') for i in results]
        self.high = [i.get('session.high') for i in results]
        self.low = [i.get('session.low') for i in results]
        self.open = [i.get('session.open') for i in results]
        self.volume =[i.get('session.volume') for i in results]
        print(self.volume)
        self.prev_close = [i.get('session.previous_close') for i in results]


        self.strike = [i.get('details.strike_price') for i in results]
        self.expiry = [i.get('details.expiration_date') for i in results]
        self.contract_type = [i.get('details.contract_type') for i in results]
        self.exercise_style = [i.get('details.exercise_style') for i in results]
        self.ticker = [i.get('details.ticker') for i in results]

  
        self.theta_values = [i.get('greeks.theta') for i in results]
        self.gamma_values = [i.get('greeks.gamma') for i in results]
        self.vega_values = [i.get('greeks.vega') for i in results]
        self.delta_values = [i.get('greeks.delta') for i in results]

        
        self.implied_volatility = [i.get('implied_volatility') for i in results]
        self.open_interest = [i.get('open_interest') for i in results]

        #last_trade = [i.get('last_trade') for i in results]
        self.sip_timestamp = [i.get('last_trade.timestamp') for i in results]
        self.conditions = [i.get('last_trade.conditions') for i in results]
        self.trade_price = [i.get('last_trade.price') for i in results]
        self.trade_size = [i.get('last_trade.size') for i in results]
        self.exchange = [i.get('last_trade.exchange') for i in results]

    
        self.ask_prices = [i.get('last_quote.ask') for i in results]
        self.bid_prices = [i.get('last_qute.bid') for i in results]
        self.bid_sizes = [i.get('last_quote.bid_size') for i in results]
        self.ask_sizes = [i.get('last_quote.ask_size') for i in results]
        self.midpoints = [i.get('last_quote.midpoint') for i in results]

        self.name = [i.get('name') for i in results]
        self.market_status = [i.get('market_status') for i in results]
        self.ticker = [i.get('ticker') for i in results]
        self.type = [i.get('type') for i in results]


        self.change_to_breakeven = [i.get('underlying_asset.change_to_break_even') for i in results]
        self.underlying_ticker = [i.get('underlying_asset.ticker') for i in results]
        if self.underlying_ticker in indices_list:
            self.underlying_price = [i.get('underlying_asset.value') for i in results]
        else:
            self.underlying_price = [i.get('underlying_asset.price') for i in results]


        # expiry_series = pd.Series(self.expiry)
        # expiry_series = pd.to_datetime(expiry_series)
        # today = pd.Timestamp(datetime.today())
        # self.days_to_expiry = (expiry_series - today).dt.days
        # self.time_value = [p - s + k if p and s and k else None for p, s, k in zip(self.trade_price, self.underlying_price, self.strike)]
        # self.moneyness = [
        #     'Unknown' if u is None else (
        #         'ITM' if (ct == 'call' and s < u) or (ct == 'put' and s > u) else (
        #             'OTM' if (ct == 'call' and s > u) or (ct == 'put' and s < u) else 'ATM'
        #         )
        #     ) for ct, s, u in zip(self.contract_type, self.strike, self.underlying_price)
        # ]


        self.data_dict = {
            
            'Change %': self.change_percent,
            'Close': self.close,
            'High': self.high,
            'Low': self.low,
            'Open': self.open,
            'Vol': self.volume,
            'Prev Close': self.prev_close,
            "cp": self.contract_type,
            'Style': self.exercise_style,
            'Exp': self.expiry,
            'Skew': self.strike,
            'Strike': self.strike,
            'Delta': self.delta,
            'Gamma': self.gamma,
            'Theta': self.theta,
            'Vega': self.vega,
            'IV': self.implied_volatility,
            'Ask': self.ask,
            'Ask Size': self.ask_size,
            'Bid': self.bid,
            'Bid Size': self.bid_size,
            'Mid': self.midpoint,
            'Timestamp': self.sip_timestamp,
            'Conditions': self.conditions,
            'Trade Price': self.trade_price,
            'Size': self.trade_size,
            'Exchange': self.exchange,
            'OI': self.open_interest,
            'Price': self.underlying_price,
            'Sym': self.underlying_ticker,
            'Name': self.name,
            'Ticker': self.ticker,
            'Types': self.type,
        }
        self.database_data_dict = {
            'days_to_expiry': self.days_to_expiry,
            'moneyness': self.moneyness,
            'time_value': self.time_value,
            'break_even_price': self.break_even_price,
            'change_percent': self.change_percent,
            'early_trading_change': self.early_trading_change,
            'early_trading_change_percent': self.early_trading_change_percent,
            'close': self.close,
            'high': self.high,
            'low': self.low,
            'open': self.open,
            'volume': self.volume,
            'prev_close': self.prev_close,
            "call_put": self.contract_type,
            'style': self.exercise_style,
            'expiry': self.expiry,
            'strike': self.strike,  # Keep this line and remove the 'Skew' entry
            'delta': self.delta,
            'gamma': self.gamma,
            'theta': self.theta,
            'vega': self.vega,
            'iv': self.implied_volatility,
            'ask': self.ask,
            'ask_size': self.ask_size,
            'bid': self.bid,
            'bid_size': self.bid_size,
            'mid': self.midpoint,
            'timestamp': self.sip_timestamp,
            'conditions': self.conditions,
            'trade_price': self.trade_price,
            'trade_size': self.trade_size,
            'trade_exchange': self.exchange,
            'oi': self.open_interest,
            'underlying_price': self.underlying_price,
            'underlying_symbol': self.underlying_ticker,
            'change_to_break_even': self.change_to_breakeven,
            'change': self.change,
            'name': self.name,
            'ticker': self.ticker,
        }


        self.skew_dict = { 
            "cp": self.contract_type,
            'iv': self.implied_volatility,
            'exp': self.expiry,
            'vol': self.volume,
            'oi': self.open_interest,
            'strike': self.strike,
}
        self.df = pd.DataFrame(self.data_dict)

        self.skew_df = pd.DataFrame(self.skew_dict)

    def __getitem__(self, index):
        return self.df[index]

    def __setitem__(self, index, value):
        self.df[index] = value
    def __iter__(self):
        # If df is a DataFrame, it's already iterable (over its column labels)
        # To iterate over rows, use itertuples or iterrows
        self.iter = self.df.itertuples()
        return self

    def __next__(self):
        # Just return the next value from the DataFrame iterator
        try:
            return next(self.iter)
        except StopIteration:
            # When there are no more rows, stop iteration
            raise StopIteration
class UniversalOptionSnapshot:
    def __init__(self, results):
        self.break_even = [float(i['break_even_price']) if 'break_even_price' is not None and 'break_even_price' in i else None for i in results]
        self.implied_volatility = [float(i['implied_volatility']) if 'implied_volatility' in i else None for i in results] 
        self.open_interest = [float(i['open_interest']) if 'open_interest' in i else None for i in results]

        day = [i['day'] if 'day' in i else 0 for i in results]
        self.volume = [float(i.get('volume',0)) for i in day]
        self.high = [float(i.get('high',0)) for i in day]
        self.low = [float(i.get('low',0)) for i in day]
        self.vwap = [float(i.get('vwap',0)) for i in day]
        self.open = [float(i.get('open',0)) for i in day]
        self.close = [float(i.get('close',0)) for i in day]
        self.change_percent= [round(float(i.get('change_percent',0))*100,2) for i in day]



        details = [i['details'] for i in results]
        self.strike = [float(i['strike_price']) if 'strike_price' in i else None for i in details]
        self.expiry = [i['expiration_date'] if 'expiration_date' in i else None for i in details]
        # Convert the expiration dates into a pandas Series
        expiry_series = pd.Series(self.expiry)
        expiry_series = pd.to_datetime(expiry_series)

        self.contract_type = [i['contract_type'] if 'contract_type' in i else None for i in details]
        self.exercise_style = [i['exercise_style'] if 'exercise_style' in i else None for i in details]
        self.ticker = [i['ticker'] if 'ticker' in i else None for i in details]

        greeks = [i['greeks'] if i['greeks'] is not None else None for i in results]
        self.theta = [round(float(i['theta']),4) if 'theta' in i else None for i in greeks]
        self.delta = [round(float(i['delta']),4) if 'delta' in i else None for i in greeks]
        self.gamma = [round(float(i['gamma']),4) if 'gamma' in i else None for i in greeks]
        self.vega = [round(float(i['vega']),4) if 'vega' in i else None for i in greeks]


        last_trade = [i['last_trade'] if i['last_trade'] is not None else None for i in results]
        self.sip_timestamp = [i['sip_timestamp'] if 'sip_timestamp' in i else None for i in last_trade]
        self.conditions = [i['conditions'] if 'conditions' in i else None for i in last_trade]
        self.conditions = [condition for sublist in self.conditions for condition in (sublist if isinstance(sublist, list) else [sublist])]
        self.trade_price = [float(i['price']) if 'price' in i else None for i in last_trade]
        self.trade_size = [float(i['size']) if 'size' in i else None for i in last_trade]
        self.exchange = [i['exchange'] if 'exchange' in i else None for i in last_trade]
        self.exchange = [OPTIONS_EXCHANGES.get(i) for i in self.exchange]

        last_quote = [i['last_quote'] if i['last_quote'] is not None else None for i in results]
        self.ask = [float(i['ask']) if 'ask' in i and i['ask'] is not None else None for i in last_quote]
        self.bid = [float(i['bid']) if 'bid' in i and i['bid'] is not None else None for i in last_quote]
        self.bid_size = [float(i['bid_size']) if 'bid_size' in i and i['bid_size'] is not None else None for i in last_quote]
        self.ask_size = [float(i['ask_size']) if 'ask_size' in i and i['ask_size'] is not None else None for i in last_quote]
        self.midpoint = [float(i['midpoint']) if 'midpoint' in i and i['midpoint'] is not None else None for i in last_quote]



        underlying_asset = [i['underlying_asset'] if i['underlying_asset'] is not None else None for i in results]
        self.change_to_breakeven = [float(i['change_to_break_even']) if 'change_to_break_even' in i else None for i in underlying_asset]
        self.underlying_price = [float(i.get('price')) if i.get('price') is not None else None for i in underlying_asset]

        self.underlying_ticker = [i['ticker'] if 'ticker' in i else None for i in underlying_asset]
        today = pd.Timestamp(datetime.today())
        
        
        expiry_series = pd.to_datetime(self.expiry)

        # Today's date
        today = pd.to_datetime(datetime.now())

        # Calculate days to expiry for each date in the series
        self.days_to_expiry_series = (expiry_series - today).days
        self.time_value = [float(p) - float(s) + float(k) if p and s and k else None for p, s, k in zip(self.trade_price, self.underlying_price, self.strike)]
        self.time_value = [round(item, 3) if item is not None else None for item in self.time_value]

        self.moneyness = [
            'Unknown' if u is None else (
                'ITM' if (ct == 'call' and s < u) or (ct == 'put' and s > u) else (
                    'OTM' if (ct == 'call' and s > u) or (ct == 'put' and s < u) else 'ATM'
                )
            ) for ct, s, u in zip(self.contract_type, self.strike, self.underlying_price)
        ]

        self.liquidity_indicator = [float(a_size) + float(b_size) if a_size is not None and b_size is not None else None for a_size, b_size in zip(self.ask_size, self.bid_size)]
        self.liquidity_indicator = [round(item, 3) if item is not None else None for item in self.liquidity_indicator]

        self.spread = [float(a) - float(b) if a is not None and b is not None else None for a, b in zip(self.ask, self.bid)]
        self.intrinsic_value = [float(u) - float(s) if ct == 'call' and u is not None and s is not None and u > s else float(s) - float(u) if ct == 'put' and u is not None and s is not None and s > u else 0.0 for ct, u, s in zip(self.contract_type, self.underlying_price, self.strike)]
        self.intrinsic_value =[round(item, 3) if item is not None else None for item in self.intrinsic_value]
        self.extrinsic_value = [float(p) - float(iv) if p is not None and iv is not None else None for p, iv in zip(self.trade_price, self.intrinsic_value)]
        self.extrinsic_value =[round(item, 3) if item is not None else None for item in self.extrinsic_value]
        self.leverage_ratio = [float(d) / (float(s) / float(u)) if d is not None and s is not None and u is not None else None for d, s, u in zip(self.delta, self.strike, self.underlying_price)]
        self.leverage_ratio = [round(item, 3) if item is not None else None for item in self.leverage_ratio]
        self.spread_pct = [(float(a) - float(b)) / float(m) * 100.0 if a is not None and b is not None and m is not None and m != 0 else None for a, b, m in zip(self.ask, self.bid, self.midpoint)]

        self.spread_pct = [round(item, 3) if item is not None else None for item in self.spread_pct]
        self.return_on_risk = [float(p) / (float(s) - float(u)) if ct == 'call' and p is not None and s is not None and u is not None and s > u else float(p) / (float(u) - float(s)) if ct == 'put' and p is not None and s is not None and u is not None and s < u else 0.0 for ct, p, s, u in zip(self.contract_type, self.trade_price, self.strike, self.underlying_price)]
        self.return_on_risk = [round(item, 3) if item is not None else None for item in self.return_on_risk]
        self.option_velocity = [float(delta) / float(p) if delta is not None and p is not None else 0.0 for delta, p in zip(self.delta, self.trade_price)]
        self.option_velocity =[round(item, 3) if item is not None else None for item in self.option_velocity]
        self.gamma_risk = [float(g) * float(u) if g is not None and u is not None else None for g, u in zip(self.gamma, self.underlying_price)]
        self.gamma_risk =[round(item, 3) if item is not None else None for item in self.gamma_risk]
        self.theta_decay_rate = [float(t) / float(p) if t is not None and p is not None else None for t, p in zip(self.theta, self.trade_price)]
        self.theta_decay_rate = [round(item, 3) if item is not None else None for item in self.theta_decay_rate]
        self.vega_impact = [float(v) / float(p) if v is not None and p is not None else None for v, p in zip(self.vega, self.trade_price)]
        self.vega_impact =[round(item, 3) if item is not None else None for item in self.vega_impact]
        self.delta_to_theta_ratio = [float(d) / float(t) if d is not None and t is not None and t != 0 else None for d, t in zip(self.delta, self.theta)]
        self.delta_to_theta_ratio = [round(item, 3) if item is not None else None for item in self.delta_to_theta_ratio]
        #option_sensitivity score - curated - finished
        self.oss = [(float(delta) if delta is not None else 0) + (0.5 * float(gamma) if gamma is not None else 0) + (0.1 * float(vega) if vega is not None else 0) - (0.5 * float(theta) if theta is not None else 0) for delta, gamma, vega, theta in zip(self.delta, self.gamma, self.vega, self.theta)]
        self.oss = [round(item, 3) for item in self.oss]
        #liquidity-theta ratio - curated - finished
        self.ltr = [liquidity / abs(theta) if liquidity and theta else None for liquidity, theta in zip(self.liquidity_indicator, self.theta)]
        #risk-reward score - curated - finished
        self.rrs = [(intrinsic + extrinsic) / (iv + 1e-4) if intrinsic and extrinsic and iv else None for intrinsic, extrinsic, iv in zip(self.intrinsic_value, self.extrinsic_value, self.implied_volatility)]
        #greeks-balance score - curated - finished
        self.gbs = [(abs(delta) if delta else 0) + (abs(gamma) if gamma else 0) - (abs(vega) if vega else 0) - (abs(theta) if theta else 0) for delta, gamma, vega, theta in zip(self.delta, self.gamma, self.vega, self.theta)]
        self.gbs = [round(item, 3) if item is not None else None for item in self.gbs]
        #options profit potential: FINAL - finished
        self.opp = [moneyness_score*oss*ltr*rrs if moneyness_score and oss and ltr and rrs else None for moneyness_score, oss, ltr, rrs in zip([1 if m == 'ITM' else 0.5 if m == 'ATM' else 0.2 for m in self.moneyness], self.oss, self.ltr, self.rrs)]
        self.opp = [round(item, 3) if item is not None else None for item in self.opp]



                


















        self.data_dict = {
            'strike': self.strike,
            'expiry': self.expiry,
            'dte': self.days_to_expiry_series,
            'time_value': self.time_value,
            'moneyness': self.moneyness,
            'liquidity_score': self.liquidity_indicator,
            "cp": self.contract_type,
            "change_ratio": self.change_percent,
            'exercise_style': self.exercise_style,
            'option_symbol': self.ticker,
            'theta': self.theta,
            'theta_decay_rate': self.theta_decay_rate,
            'delta': self.delta,
            'delta_theta_ratio': self.delta_to_theta_ratio,
            'gamma': self.gamma,
            'gamma_risk': self.gamma_risk,
            'vega': self.vega,
            'vega_impact': self.vega_impact,
            'timestamp': self.sip_timestamp,
            'oi': self.open_interest,
            'open': self.open,
            'high': self.high,
            'low': self.low,
            'close': self.close,
            'intrinstic_value': self.intrinsic_value,
            'extrinsic_value': self.extrinsic_value,
            'leverage_ratio': self.leverage_ratio,
            'vwap':self.vwap,
            'conditions': self.conditions,
            'price': self.trade_price,
            'trade_size': self.trade_size,
            'exchange': self.exchange,
            'ask': self.ask,
            'bid': self.bid,
            'spread': self.spread,
            'spread_pct': self.spread_pct,
            'iv': self.implied_volatility,
            'bid_size': self.bid_size,
            'ask_size': self.ask_size,
            'vol': self.volume,
            'mid': self.midpoint,
            'change_to_breakeven': self.change_to_breakeven,
            'underlying_price': self.underlying_price,
            'ticker': self.underlying_ticker,
            'return_on_risk': self.return_on_risk,
            'velocity': self.option_velocity,
            'sensitivity': self.oss,
            'greeks_balance': self.gbs,
            'opp': self.opp
            
        }


        # Create DataFrame from data_dict
        self.df = pd.DataFrame(self.data_dict)
    def __repr__(self) -> str:
        return f"UniversalOptionSnapshot(break_even={self.break_even}, \
                implied_volatility={self.implied_volatility},\
                open_interest ={self.open_interest}, \
                change={self.exchange}, \
                expiry={self.expiry}, \
                ticker={self.ticker} \
                contract_type={self.contract_type}, \
                exercise_style={self.exercise_style}, \
                theta={self.theta}, \
                delta={self.delta}, \
                gamma={self.gamma}, \
                vega={self.vega}, \
                sip_timestamp={self.sip_timestamp}, \
                conditions={self.conditions}, \
                trade_price={self.trade_price}, \
                trade_size={self.trade_size}, \
                exchange={self.exchange}, \
                ask={self.ask}, \
                bid={self.bid}, \
                bid_size={self.bid_size}, \
                ask_size={self.ask_size}, \
                midpoint={self.midpoint}, \
                change_to_breakeven={self.change_to_breakeven}, \
                underlying_price={self.underlying_price}, \
                underlying_ticker={self.underlying_ticker})"
    
    def __getitem__(self, index):
        return self.df[index]

    def __setitem__(self, index, value):
        self.df[index] = value
    def __iter__(self):
        # If df is a DataFrame, it's already iterable (over its column labels)
        # To iterate over rows, use itertuples or iterrows
        self.iter = self.df.itertuples()
        return self

    def __next__(self):
        # Just return the next value from the DataFrame iterator
        try:
            return next(self.iter)
        except StopIteration:
            # When there are no more rows, stop iteration
            raise StopIteration

class CallsOrPuts:
    def __init__(self, data):
        self.cfi = [i['cfi'] if 'cfi' in i else None for i in data]
        self.contract_type = [i['contract_type'] if 'contract_type' in i else None for i in data]
        self.exercise_style = [i['exercise_style'] if 'exercise_style' in i else None for i in data]
        self.expiration_date = [i['expiration_date'] if 'expiration_date' in i else None for i in data]
        self.primary_exchange = [i['primary_exchange'] if 'primary_exchange' in i else None for i in data]
        self.shares_per_contract = [i['shares_per_contract'] if 'shares_per_contract' in i else None for i in data]
        self.strike_price = [i['strike_price'] if 'strike_price' in i else None for i in data]
        self.ticker = [i['ticker'] if 'ticker' in i else None for i in data]
        self.underlying_ticker = [i['underlying_ticker'] if 'underlying_ticker' in i else None for i in data]


        self.data_dict = { 
            'ticker': self.ticker,
            'strike': self.strike_price,
            'expiry': self.expiration_date

        }


        self.df = pd.DataFrame(self.data_dict).sort_values(by='expiry')

class MultipleUniversalOptionSnapshot:
    def __init__(self, results):
        self.break_even = results.get('break_even_price', None)
     
        self.implied_volatility = results.get('implied_volatility', None)
        self.open_interest = results.get('open_interest', None)

        day = results.get('day', None)
        self.volume = day.get('volume', None)
        self.high = day.get('high', None)
        self.low = day.get('low', None)
        self.vwap = day.get('vwap', None)
        self.open = day.get('open', None)
        self.close = day.get('close', None)




        details = results.get('details', None)
        self.strike = details.get('strike_price', None)
        self.expiry =  details.get('expiration_date', None)
        self.contract_type =  details.get('contract_type', None)
        self.exercise_style =  details.get('exercise_style', None)
        self.ticker =  details.get('ticker', None)

        greeks = results.get('greeks', None)
        self.theta = greeks.get('theta', None)
        self.delta = greeks.get('delta', None)
        self.gamma = greeks.get('gamma', None)
        self.vega = greeks.get('vega', None)


        last_trade = results.get('last_trade', None)
        self.sip_timestamp = last_trade.get('sip_timestamp', None)
        self.conditions = last_trade.get('conditions', None)
        self.trade_price = last_trade.get('price', None)
        self.trade_size = last_trade.get('size', None)
        self.exchange = last_trade.get('exchange', None)

        last_quote = results.get('last_quote', None)
        self.ask = last_quote.get('ask', None)
        self.bid = last_quote.get('bid', None)
        self.bid_size = last_quote.get('bid_size', None)
        self.ask_size = last_quote.get('ask_size', None)
        self.midpoint = last_quote.get('midpoint', None)


        underlying_asset = results.get('underlying_asset', None)
        self.change_to_breakeven = underlying_asset.get('change_to_breakeven', None)
        self.underlying_price = underlying_asset.get('underlying_price', None)
        self.underlying_ticker = underlying_asset.get('underlying_ticker', None)

        self.data_dict = {
            'strike': self.strike,
            'exp': self.expiry,
            'type': self.contract_type,
            'exercise_style': self.exercise_style,
            'ticker': self.ticker,
            'theta': self.theta,
            'delta': self.delta,
            'gamma': self.gamma,
            'vega': self.vega,
            'sip_timestamp': self.sip_timestamp,
            'open': self.open,
            'high': self.high,
            'low': self.low,
            'close': self.close,
            'vwap':self.vwap,
            'conditions': self.conditions,
            'price': self.trade_price,
            'Size': self.trade_size,
            'exchange': self.exchange,
            'ask': self.ask,
            'bid': self.bid,
            'IV': self.implied_volatility,
            'bid_size': self.bid_size,
            'ask_size': self.ask_size,
            'vol': self.volume,
            'entryCost': self.midpoint,
            'change_to_breakeven': self.change_to_breakeven,
            'price': self.underlying_price,
            'sym': self.underlying_ticker
        }

        self.df = pd.DataFrame(self.data_dict)



import os
from dotenv import load_dotenv
from discord_.discord_sdk import DiscordSDK
load_dotenv()



class DiscordArchitect(DiscordSDK):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)




    



from apis.polygonio.async_polygon_sdk import Polygon
import os
import asyncio
poly = Polygon()


# Creating a database configuration dictionary using os.environ.get for environment variables
db_config = {
    "DB_HOST": os.environ.get('DB_HOST'),
    "DB_PASSWORD": os.environ.get('DB_PASSWORD'),
    "DB_NAME": os.environ.get('DB_NAME'),
    "DB_PORT": os.environ.get('DB_PORT', 5432),
    "DB_USER": os.environ.get('DB_USER')
}



async def main():
    x = await poly.aggregates('I:SPX', timespan='second', date_from='2023-10-27', date_to='2023-10-27', limit=50000)

    await x.create_pool(db_config=db_config)
    await x.create_table()
    
    # Get the data dictionary
    data_dict = x.data_dict

    # Transform data_dict into a list of dictionaries for insertion
    data_dicts = []
    for o, h, l, c, t in zip(data_dict['Open'], data_dict['High'], data_dict['Low'], data_dict['Close'], data_dict['Timestamp']):
        single_record = {
            'Ticker': 'I:SPX',  # Or data_dict['Ticker'] if it's in the dictionary
            'Open': o,
            'High': h,
            'Low': l,
            'Close': c,
            'Timestamp': t
        }
        data_dicts.append(single_record)

    # Insert the list of dictionaries into the database
    await x.insert_data(data_dicts)

asyncio.run(main())


import os

from dotenv import load_dotenv
load_dotenv()
import sys
from pathlib import Path
# Add the project directory to the sys.path
project_dir = str(Path(__file__).resolve().parents[1])
if project_dir not in sys.path:
    sys.path.append(project_dir)
from database_ import DatabaseManager
import asyncio


class OptionScripts():
    def __init__(self):
        self.db = DatabaseManager(host='localhost', user='chuck', port=5432, password='fud', database='markets')



        
    async def yield_ticker(self):
        await self.db.connect()
        buffer = []
        while True:
            query = """SELECT DISTINCT option_symbol FROM options_data WHERE expiry > '2024-01-06';"""
            async for record in self.db.fetch_iter(query):
                ticker = record['option_symbol']


                buffer.append(ticker)
                if len(buffer) >= 250:
                    yield ','.join(buffer)
                    buffer = []


scripts = OptionScripts()
async def main():
    async for ticker in scripts.yield_ticker():
        print(ticker)

asyncio.run(main())


import os
from dotenv import load_dotenv
import requests
import base64
load_dotenv()

from openai import OpenAI


class OpenAISDK:
    def __init__(self):
        self.key = os.environ.get('YOUR_OPENAI_KEY')
        self.client = OpenAI(api_key=self.key)

    def analyze_stock(self, url):
        

        response = self.client.chat.completions.create(
        model="gpt-4-vision-preview",
        messages=[
            {
            "role": "user",
            "content": [
                {"type": "text", "text": "Examine the validity of this image by reading the text if any. If no text-  provide an overall analysis."},
                {
                "type": "image_url",
                "image_url": {
                    "url": f"{url}",
                    "detail": "high"
                },
                },
            ],
            }
        ],
        max_tokens=300,
        )

        print(response.choices[0].message.content)
        return response.choices[0].message.content




import os
from dotenv import load_dotenv
load_dotenv()
import disnake
from disnake.ext import commands
from apis.y_finance.yf_sdk import yfSDK



class yfCOG(commands.Cog):
    def __init__(self, bot):
        self.bot=bot
        self.sdk = yfSDK()



    @commands.slash_command()
    async def info(self, inter):
        pass


    @info.sub_command()
    async def mf(self, inter: disnake.AppCmdInter, ticker: str):
        """Gets mutual fund holdings for a ticker - top 10"""
        await inter.response.defer()
        try:
            data = self.sdk.mutual_fund_holders(ticker)
            
            filename = f'data/yf_{ticker}_mf_holders.csv'
            data.to_csv(filename)
            
            embed = disnake.Embed(
                title=f"Mutual Fund Holders - {ticker}", 
                description=f"Your Download is Ready!", 
                color=disnake.Colour.dark_teal()
            )
            embed.set_footer(text=f'Implemented by FUDSTOP')
            
            await inter.edit_original_message(embed=embed)
            await inter.send(file=disnake.File(filename), view=YFView(ticker))
        except Exception as e:
            await inter.edit_original_message(f"An error occurred: {e}")


    @info.sub_command()
    async def balance(self, inter:disnake.AppCmdInter, ticker: str):
        """Gets balance sheet information for a ticker"""
        await inter.response.defer()
        try:
            data = self.sdk.balance_sheet(ticker)
            
            filename = f'data/yf_balance_sheet.csv'
            data.to_csv(filename)
            
            await inter.edit_original_message(f"Balance Sheet for {ticker}:")
            await inter.edit_original_message(file=disnake.File(filename), view=YFView(ticker))
        except Exception as e:
            await inter.edit_original_message(f"An error occurred: {e}")

    @info.sub_command()
    async def cashflow(self, inter:disnake.AppCmdInter, ticker: str):
        """Gets cash flow information for a ticker"""
        await inter.response.defer()
        try:
            data = self.sdk.get_cash_flow(ticker)
            
            filename = f'data/yf_cash_flow.csv'
            data.to_csv(filename)
            
            await inter.edit_original_message(f"Cash Flow for {ticker}:")
            await inter.edit_original_message(file=disnake.File(filename), view=YFView(ticker))
        except Exception as e:
            await inter.edit_original_message(f"An error occurred: {e}")

    @info.sub_command()
    async def financials(self, inter:disnake.AppCmdInter, ticker: str):
        """Gets all financials for a ticker"""
        await inter.response.defer()
        try:
            data = self.sdk.financials(ticker)
            
            filename = f'data/yf_financials.csv'
            data.to_csv(filename)
            
            await inter.edit_original_message(f"Financials for {ticker}:")
            await inter.edit_original_message(file=disnake.File(filename), view=YFView(ticker))
        except Exception as e:
            await inter.edit_original_message(f"An error occurred: {e}")


    # Command for the income_statement method within yfSDK
    @info.sub_command()
    async def statement(self, inter:disnake.AppCmdInter, ticker: str, frequency: str = 'quarterly', pretty: bool = False, as_dict: bool = False):
        """Gets the income statement for a ticker"""
        await inter.response.defer()
        data = self.sdk.income_statement(ticker, frequency=frequency, pretty=pretty, as_dict=as_dict)
        filename = f'data/yf_income_statement.csv'
        data.to_csv(filename)
        await inter.edit_original_message(file=disnake.File(filename), view=YFView(ticker))

    # Command for the get_info method within yfSDK
    @info.sub_command()
    async def infos(self, inter:disnake.AppCmdInter, ticker: str):
        """Returns a large dictionary of information for a ticker"""
        await inter.response.defer()
        data = self.sdk.get_info(ticker)
        filename = f'data/yf_info.csv'
        data.to_csv(filename)
        await inter.edit_original_message(file=disnake.File(filename), view=YFView(ticker))

    # Command for the institutional_holdings method within yfSDK
    @info.sub_command()
    async def whales(self, inter:disnake.AppCmdInter, ticker: str):
    
        """Gets institutional holdings for a ticker"""
        await inter.response.defer()
        data = self.sdk.institutional_holdings(ticker)
        filename = f'data/yf_institutional_holdings.csv'
        data.to_csv(filename)
        await inter.edit_original_message(file=disnake.File(filename), view=YFView(ticker))


    @info.sub_command()
    async def div(self, inter:disnake.AppCmdInter, ticker: str):
        """Gets dividends for a ticker - if any."""
        await inter.response.defer()
        data = self.sdk.dividends(ticker)
        filename = f'data/dividends.csv'
        data.to_csv(filename)
        await inter.edit_original_message(file=disnake.File(filename), view=YFView(ticker))


    @info.sub_command()
    async def allinfo(self, inter:disnake.AppCmdInter, ticker: str):
        """Gets all relevant company data for a ticker."""
        await inter.response.defer()
        data = self.sdk.fast_info(ticker)
        filename = f'data/fast_info.csv'
        data.to_csv(filename)
        await inter.edit_original_message(file=disnake.File(filename), view=YFView(ticker))


    @info.sub_command()
    async def candles(self, inter:disnake.AppCmdInter, *, ticker: str):
        """Gets all candlestick data for a ticker"""
        await inter.response.defer()
        data = self.sdk.get_all_candles(ticker)
        filename = f'data/all_candles.csv'
        data.to_csv(filename)
        await inter.edit_original_message(file=disnake.File(filename), view=YFView(ticker))


    # Command for the news method within yfSDK
    @info.sub_command()
    async def news(self, inter:disnake.AppCmdInter, ticker: str):
        """Gets ticker news"""
        await inter.response.defer()
        data = self.sdk.news(ticker)
        filename = f'data/yf_news.csv'
        data.to_csv(filename)
        await inter.edit_original_message(file=disnake.File(filename), view=YFView(ticker))

    # Command for the atm_calls method within yfSDK
    @info.sub_command()
    async def calls(self, inter:disnake.AppCmdInter, ticker: str):
        """Gets at the money calls for a ticker"""
        await inter.response.defer()
        data = self.sdk.atm_calls(ticker)
        data.to_csv('data/atm_calls.csv')
        filename = f'data/atm_calls.csv'
        data.to_csv(filename, index=False)
        await inter.edit_original_message(file=disnake.File(filename), view=YFView(ticker))

    # Command for the atm_calls method within yfSDK
    @info.sub_command()
    async def puts(self, inter:disnake.AppCmdInter, ticker: str):
        """Gets at the money puts for a ticker"""
        await inter.response.defer()
        data = self.sdk.atm_puts(ticker)
        data.to_csv('data/atm_puts.csv')
        filename = f'data/atm_puts.csv'
        data.to_csv(filename, index=False)
        await inter.edit_original_message(file=disnake.File(filename), view=YFView(ticker))

yf = yfCOG(commands.Bot)

class YFView(disnake.ui.View):
    def __init__(self, ticker=None):
        self.ticker=ticker
        super().__init__(timeout=None)

        self.add_item(YFSelect(ticker=self.ticker))



class YFSelect(disnake.ui.Select):
    def __init__(self, ticker=None):
        self.ticker=ticker

        super().__init__(placeholder='Select A Command -->',
                         min_values=1,
                         max_values=1,
                         custom_id='YFSELECT',
                         options = [ 
                             disnake.SelectOption(label='Mutual Fund Holders',value='0',description='View top mutual fund holders for a ticker'),
                             disnake.SelectOption(label='Balance Sheet',value='1',description='Get balance sheet info for a ticker.'),
                             disnake.SelectOption(label='Cash Flow',value='2',description='Get cash flow info for a ticker.'),
                            disnake.SelectOption(label='Income Statement',value='3',description='Get the income statement for a ticker.'),
                             disnake.SelectOption(label='All Financials',value='4',description='Get all company financials for a ticker.'),
                            
                             disnake.SelectOption(label='Ticker Info',value='5',description='Get a vast amount of company info for a ticker.'),
                             disnake.SelectOption(label='Institutions',value='6',description='Get top institutional holdings for a ticker.'),
                             disnake.SelectOption(label='Dividends',value='7',description='Get dividend information for a ticker.'),
                             disnake.SelectOption(label='All Info',value='8',description='Get all information for a ticker.'),
                             disnake.SelectOption(label='News', value='9', description=f'Get recent news for a ticker.'),
                             disnake.SelectOption(label='ATM Calls', value='10', description=f'Get at the money calls for a ticker.'),
                             disnake.SelectOption(label='ATM Puts', value='11', description='Get at the money puts for a ticker.'),
                             disnake.SelectOption(label='All Candles', value='12', description=f"Get all candlestick data for a ticker.")
                         ])
        
    async def callback(self, inter: disnake.AppCmdInter):
        if self.values[0] == '0':
            await yf.mf(inter,self.ticker)
        elif self.values[0] == '1':
            await yf.balance(inter, self.ticker)

        elif self.values[0] == '2':
            await yf.cashflow(inter, self.ticker)
        elif self.values[0] == '3':
            await yf.statement(inter, self.ticker)
        elif self.values[0] == '4':
            await yf.financials(inter, self.ticker)
        elif self.values[0] == '5':
            await yf.infos(inter, self.ticker)
        elif self.values[0] == '6':
            await yf.whales(inter, self.ticker)
        elif self.values[0] == '7':
            await yf.div(inter, self.ticker)
        elif self.values[0] == '8':
            await yf.allinfo(inter, self.ticker)
        elif self.values[0] == '9':
            await yf.news(inter,self.ticker)
        elif self.values[0] == '10':
            await yf.calls(inter,self.ticker)
        elif self.values[0] == '11':
            await yf.puts(inter,self.ticker)
        elif self.values[0] == '12':
            await yf.candles(inter,self.ticker)



def setup(bot: commands.Bot):
    bot.add_cog(yfCOG(bot))


    print(f'YF COG - READY')



from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
def take_screenshot(url, file_name):
    """
    Opens the URL in a headless Chrome, clicks the specified button, selects the stars, and takes a screenshot.
    :param url: URL of the webpage to take a screenshot of
    :param file_name: Name of the file to save the screenshot
    :param chrome_driver_path: Path to the chromedriver executable
    """
    # Chrome options for headless browser
    chrome_options = Options()
    chrome_options.add_argument("--headless")
    chrome_options.add_argument("--disable-gpu")
    chrome_options.add_argument("--no-sandbox")
    chrome_options.add_argument("--disable-web-security")

    # Initialize the driver
    driver = webdriver.Chrome(options=chrome_options)
    driver.implicitly_wait(10)  # Wait for elements to be present

    try:
        # Open the URL
        driver.get(url)

        # Wait for the button to be clickable and click it
        button = WebDriverWait(driver, 20).until(
            EC.element_to_be_clickable((By.CSS_SELECTOR, "button#ctl00_ContentPlaceHolder1_ctl02_Button1"))
        )
        button.click()

        # Wait for the stars to be clickable and click the third star
        third_star = WebDriverWait(driver, 20).until(
            EC.element_to_be_clickable((By.CSS_SELECTOR, "span.d-none.d-lg-inline + span + span"))
        )
        third_star.click()

        # Take the screenshot
        driver.save_screenshot(file_name)

    except Exception as e:
        print(f"An error occurred: {e}")
    finally:
        driver.quit()

# Example usage
take_screenshot('https://tradingeconomics.com/calendar', 'screenshot.png')


import disnake

from disnake.ext import commands
from apis.webull.webull_trading import WebullTrading




class WebullCOG(commands.Cog):
    def __init__(self, bot):
        self.bot=bot
        self.wt = WebullTrading()



    @commands.slash_command()
    async def webull(self, inter):
        pass




    @webull.sub_command()
    async def multi_quote(self, inter:disnake.AppCmdInter, tickers:str):
        """Stream quotes for a comma separated list of tickers .. e.g. MSFT,AAPL,TSLA.."""
        await inter.response.defer()
        all_quotes = []
        async for quotes in self.wt.multi_quote(tickers):

            # Split the quotes string by '|', assuming each piece of information is separated by '|'
            quote_details = quotes.split(' | ')

            # Join the split quote details with a newline character
            formatted_quote = '\n'.join(quote_details)

            # Append the formatted quote to the all_quotes list
            all_quotes.append(formatted_quote)

        # Join all quotes with two newlines for clear separation between each ticker's quotes
        final_output = "\n\n".join(all_quotes)
        await inter.edit_original_message(f"# > {final_output}")



def setup(bot:commands.Bot):
    bot.add_cog(WebullCOG(bot))
    print('Webull ready!')



from typing import List
from polygon.websocket import WebSocketMessage, EquityAgg,EquityQuote,EquityTrade, Market
from apis.polygonio.mapping import stock_condition_dict, STOCK_EXCHANGES, TAPES, quote_conditions, indicators
from datetime import datetime
import asyncio

from list_sets.ticker_lists import most_active_tickers




batch_data_aggs = []
batch_data_trades = []
batch_data_quotes = []
async def handle_stock_msg(msgs: List[WebSocketMessage], data_queue: asyncio.Queue, db=None):

    global batch_data_aggs, batch_data_trades, batch_data_quotes


    for m in msgs:
        if m.symbol in most_active_tickers:




            if isinstance(m, EquityAgg):
                data = {
                    'type': 'EquityAgg',
                    'stock_symbol': m.symbol,
                    'close_price': m.close,
                    'high_price': m.high,
                    'low_price': m.low,
                    'open_price': m.open,
                    'volume': m.volume,
                    'official_open': m.official_open_price,
                    'accumulated_volume': m.accumulated_volume,
                    'vwap_price': m.vwap,
                    'agg_timestamp': datetime.fromtimestamp(m.end_timestamp / 1000.0) if m.end_timestamp is not None else None
                }


                data = {
                    'ticker': m.symbol,
                    'close_price': m.close,
                    'high_price': m.high,
                    'low_price': m.low,
                    'open_price': m.open,
                    'volume': m.volume,
                    'official_open': m.official_open_price,
                    'accumulated_volume': m.accumulated_volume,
                    'vwap_price': m.vwap,
                    'agg_timestamp': datetime.fromtimestamp(m.end_timestamp / 1000.0) if m.end_timestamp is not None else None
                }

                asyncio.create_task(data_queue.put(data))
                # if db is not None:
                #     await db.save_structured_message(data, 'stock_aggs')
                


            elif isinstance(m, EquityTrade):

                data = { 
                    'type': 'EquityTrade',
                    'stock_symbol': m.symbol,
                    'trade_exchange': STOCK_EXCHANGES.get(m.exchange),
                    'trade_price': m.price,
                    'trade_size': m.size,
                    'trade_conditions': [stock_condition_dict.get(condition) for condition in m.conditions] if m.conditions is not None else [],
                    'trade_timestamp': datetime.fromtimestamp(m.timestamp / 1000.0) if m.timestamp is not None else None
                }


                asyncio.create_task(data_queue.put(data))
                # if db is not None:
                #     await db.save_structured_message(data, 'equity_trades')


            elif isinstance(m, EquityQuote):
                timetamp = datetime.fromtimestamp(m.timestamp / 1000.0).strftime('%Y-%m-%d %H:%M:%S')
                timestamp_obj = datetime.strptime(timetamp, '%Y-%m-%d %H:%M:%S')
                data = {
                'type': 'EquityQuote',
                'quote_symbol': m.symbol,
                'ask': m.ask_price,
                'bid':m.bid_price,
                'ask_size': m.ask_size,
                'bid_size':m.bid_size,
                'indicator': [indicators.get(indicator) for indicator in m.indicators] if m.indicators is not None else [],
                'condition':quote_conditions.get(m.condition),

                
                'ask_exchange':STOCK_EXCHANGES.get(m.ask_exchange_id),
                'bid_exchange':STOCK_EXCHANGES.get(m.bid_exchange_id),
                
                'timestamp': timestamp_obj,
                'tape': TAPES.get(m.tape)}
                asyncio.create_task(data_queue.put(data))

                # if db is not None:
                #     await db.insert_equity_quote(data)










import sys
from pathlib import Path

# Add the project directory to the sys.path
project_dir = str(Path(__file__).resolve().parents[1])
if project_dir not in sys.path:
    sys.path.append(project_dir)
import asyncio
import pandas as pd
from dotenv import load_dotenv
from apis.webull.webull_options import WebullOptions
from apis.helpers import get_human_readable_string

load_dotenv()
import os
options = WebullOptions(connection_string=os.environ.get('WEBULL_STRING'))

import asyncio
import pandas as pd
from dotenv import load_dotenv
from apis.webull.webull_options import WebullOptions
from apis.helpers import get_human_readable_string

load_dotenv()

options = WebullOptions(connection_string=os.environ.get('WEBULL_STRING'))

async def fetch_volume_analysis(option_id, ticker):
    try:
        # Asynchronous database query
        symbol_query = f"SELECT option_symbol FROM wb_opts WHERE ticker_id = {option_id}"
        symbol_result = await options.fetch(symbol_query)

        if symbol_result and 'option_symbol' in symbol_result[0]:
            option_symbol = symbol_result[0]['option_symbol']
            components = get_human_readable_string(option_symbol)
            print(components)
            # Asynchronous fetch for volume analysis
            data_frame = await options.fetch_volume_analysis(option_symbol=option_symbol, id=option_id, underlying_ticker=ticker)
            if data_frame is not None:
                data_frame['option_symbol'] = option_symbol
                data_frame['underlying_symbol'] = components.get('underlying_symbol')
                data_frame['strike_price'] = components.get('strike_price')
                data_frame['call_put'] = components.get('call_put')
                data_frame['expire_date'] = components.get('expiry_date')
                await options.batch_insert_dataframe(data_frame,table_name='vol_anal', unique_columns='option_symbol')
                return data_frame
    except Exception as e:
        print(f"Error processing option ID {option_id}: {e}")

async def process_batch(batch, ticker):
    return await asyncio.gather(*[fetch_volume_analysis(option_id, ticker) for option_id in batch])

async def main(ticker):
    

    option_ids = await options.get_option_id_for_symbol(ticker)

    batch_size = 100  # Adjust based on performance
    batches = [option_ids[i:i + batch_size] for i in range(0, len(option_ids), batch_size)]

    all_dataframes = []
    for batch in batches:
        dataframes = await process_batch(batch, ticker)
        all_dataframes.extend(df for df in dataframes if df is not None)

    await options.batch_insert_dataframe(all_dataframes,table_name='vol_anal', unique_columns='option_symbol')
    # final_dataframe = pd.concat(all_dataframes, ignore_index=True)
    # final_dataframe.to_csv('SPY_OPTIONS.csv', index=False)


async def run_main():
    await options.connect()
    tasks = [main(i) for i in options.most_active_tickers]

    await asyncio.gather(*tasks)

asyncio.run(run_main())



import sys
import os

sys.path.append(os.path.dirname(os.path.abspath(__file__)))


from typing import List
import disnake

class AlertMenus(disnake.ui.View):
    def __init__(self, embeds: List[disnake.Embed] = None, assistant_id=None, thread_id=None):
        
        super().__init__(timeout=None)
        self.assistant_id = assistant_id
        self.thread_id = thread_id
        self.embeds = embeds if embeds is not None else []
        self.embed_count = 0
        self.prev_page.disabled = True
        self.count = 0

        if self.embeds:
            for i, embed in enumerate(self.embeds):
                embed.set_footer(text=f"Page {i + 1} of {len(self.embeds)}")

        if self.embeds and assistant_id is not None and thread_id is not None:
            for i, embed in enumerate(self.embeds):
                embed.set_footer(text=f"Page {i + 1} of {len(self.embeds)} | thread: {thread_id} | assistant | {assistant_id}")  



    @disnake.ui.button(
        style=disnake.ButtonStyle.red,
        custom_id=f"persistent_view:prevqwfpage_{str(disnake.Member)}aq2wfwa",
        row=4,
        label=f"🇵 🇷 🇪 🇻"

    )
    async def prev_page(  # pylint: disable=W0613
        self,
        button: disnake.ui.Button,
        interaction: disnake.MessageInteraction,
    ):
        # Decrements the embed count.
        self.embed_count -= 1

        # Gets the embed object.
        embed = self.embeds[self.embed_count]

        # Enables the next page button and disables the previous page button if we're on the first embed.
        self.next_page.disabled = False

        await interaction.response.edit_message(embed=embed, view=self)


    @disnake.ui.button(
        style=disnake.ButtonStyle.red,
        custom_id=f"persistent_view:nextpage_{str(disnake.Member)}awfawwa",
        label=f"🇳 🇪 🇽 🇹",
        row=4
    )
    async def next_page(
        self,
        button: disnake.ui.Button,
        interaction: disnake.MessageInteraction,
    ):
        # Checks if self.embed_count is within the valid range
        if 0 <= self.embed_count < len(self.embeds):
            # Increments the embed count
            self.embed_count += 1

            # Gets the embed object
            embed = self.embeds[self.embed_count]

            # Enables the previous page button and disables the next page button if we're on the last embed
            self.prev_page.disabled = False
            if self.embed_count == len(self.embeds) - 1:
                self.next_page.disabled = True

            await interaction.response.edit_message(embed=embed, view=self)


class PageSelect(disnake.ui.Select):
    def __init__(self, embeds: List[disnake.Embed]):
        # Create options using the title of each embed
        options = [
            disnake.SelectOption(
                label=embed.title,  # Use the title of the embed as the label
                value=str(i)  # Keep the value as a string representation of the index
            ) for i, embed in enumerate(embeds)
        ]

        # Initialize the Select menu with the options
        super().__init__(placeholder="Choose a page...", min_values=1, max_values=1, options=options)

        super().__init__(
            custom_id="page_selector1",
            placeholder="Pages 1-25",
            min_values=1,
            max_values=1,
            options=options,
            row=0
        )
        
        self.embeds = embeds

    async def callback(self, interaction: disnake.Interaction):
        await interaction.response.edit_message(embed=self.embeds[int(self.values[0])])


import os
from dotenv import load_dotenv
load_dotenv()
from fudstop.apis.polygonio.polygon_options import PolygonOptions
from fudstop.apis.polygonio.async_polygon_sdk import Polygon
import numpy as np
import pandas as pd
import asyncio
import requests
from datetime import datetime, timezone, timedelta
poly = Polygon(connection_string=os.environ.get('POLYGON_STRING'))

ticker='AMZN'
timespan='day'
window=50
limit=100
timeframe='quarterly'

multiplier = 1
date_from = '2023-12-01'
date_to = '2023-12-29'


class PolygonMaster:
    def __init__(self, **kwargs):
        self.kwargs = kwargs
        # Process each kind of results if it's provided
        if 'macd_results' in kwargs:
            self.process_macd_results(kwargs['macd_results'])
        if 'rsi_results' in kwargs:
            self.process_rsi_results(kwargs['rsi_results'])

        if 'ema_results' in kwargs:
            self.process_ema_results(kwargs['ema_results'])
        
        if 'sma_results' in kwargs:
            self.process_sma_results(kwargs['sma_results'])

        if 'ticker_news_results' in kwargs:
            self.process_ticker_news_results(kwargs['ticker_news_results'])

        if 'aggregates_results' in kwargs:
            self.process_aggregates_results(kwargs['aggregates_results'])

        if 'all_stock_snapshot_results' in kwargs:
            self.process_all_stock_snapshot_results(kwargs['all_stock_snapshot_results'])
        
        if 'financials_results' in kwargs:
            self.process_annual_financial_results(kwargs['financials_results'])

        self.process_api_results(kwargs)

    def format_large_number(self, value):
        """
        Format large numbers into a readable form, handling both positive and negative values.
        """
        if pd.isnull(value):
            return value
        value = float(value)
        is_negative = value < 0
        if is_negative:
            value = -value
        if value >= 1e12:
            formatted_value = f'{value / 1e12:.2f}T'
        elif value >= 1e9:
            formatted_value = f'{value / 1e9:.2f}B'
        elif value >= 1e6:
            formatted_value = f'{value / 1e6:.2f}M'
        elif value >= 1e3:
            formatted_value = f'{value / 1e3:.2f}K'
        else:
            formatted_value = str(value)
        return f'-{formatted_value}' if is_negative else formatted_value

    def format_large_numbers_in_dataframe(self, df):
        """
        Automatically formats all numeric columns in a DataFrame to readable large numbers.
        """
        formatted_df = df.copy()
        numeric_columns = df.select_dtypes(include=[np.number]).columns
        for column in numeric_columns:
            formatted_df[column] = formatted_df[column].apply(self.format_large_number)
        return formatted_df


    def process_api_results(self, api_results):
        for result_type, result_data in api_results.items():
            if result_data is not None:
                processing_method = getattr(self, f'process_{result_type}', None)
                if processing_method:
                    processing_method(result_data)


    def process_rsi_results(self, rsi_results):
        self.underlying = rsi_results.get('underlying')
        self.rsi_values = rsi_results.get('values')
                
        self.rsi_timestamp = []
        for i in self.rsi_values:
            timestamp = i.get('timestamp')

            if timestamp:
                try:
                    # Convert timestamp from milliseconds to seconds
                    timestamp = float(timestamp) / 1000

                    # Convert Unix timestamp to datetime and adjust timezone
                    dt_object = datetime.fromtimestamp(timestamp, tz=timezone.utc)
                    dt_object = dt_object.astimezone(timezone(timedelta(hours=-5)))
                    self.rsi_timestamp.append(dt_object.strftime('%Y-%m-%d %H:%M:%S'))
                except Exception as e:
                    self.rsi_timestamp.append(f'Error: {e}')
            else:
                self.rsi_timestamp.append('Timestamp Missing')
        self.rsi_value = [float(i.get('value')) if i.get('value') is not None else 'N/A' for i in self.rsi_values]
        self.rsi_data_dict = { 
            'rsi_value': self.rsi_value,
            'rsi_timestamp': self.rsi_timestamp,
            'rsi_timespan': timespan
        }

        self.rsi_as_dataframe = pd.DataFrame(self.rsi_data_dict)
        if 'ticker' in self.kwargs:
            # Add the ticker as a new column
            self.rsi_as_dataframe['ticker'] = self.kwargs['ticker']
            



    def process_macd_results(self, macd_results):
        self.underlying = macd_results.get('underlying')
        self.macd_values = macd_results.get('values')
        self.macd_timestamp = []
        for i in self.macd_values:
            timestamp = i.get('timestamp')
            if timestamp:
                try:
                    # Convert from milliseconds to seconds if needed
                    timestamp = float(timestamp)
                    if len(str(int(timestamp))) == 13:  # Check if timestamp is in milliseconds
                        timestamp /= 1000

                    # Convert Unix timestamp to datetime and adjust timezone
                    formatted_timestamp = (
                        datetime.fromtimestamp(timestamp, tz=timezone.utc)
                        .astimezone(timezone(timedelta(hours=-5)))  # Eastern Time (ET) UTC-5
                        .strftime('%Y-%m-%d %H:%M:%S')
                    )
                    self.macd_timestamp.append(formatted_timestamp)
                except Exception as e:
                    self.macd_timestamp.append(f'Error: {e}')
            else:
                self.macd_timestamp.append('N/A')

        self.macd = [float(i.get('value')) if i.get('value') is not None else 'N/A' for i in self.macd_values]
        self.signal = [float(i.get('signal')) if i.get('signal') is not None else 'N/A' for i in self.macd_values]
        self.histogram = [float(i.get('histogram')) if i.get('histogram') is not None else 'N/A' for i in self.macd_values]

        self.macd_data_dict = { 
            'macd': self.macd,
            'histogram': self.histogram,
            'signal': self.signal,
            'timestamp': self.macd_timestamp
        }
        self.macd_as_dataframe = pd.DataFrame(self.macd_data_dict)

        if 'ticker' in self.kwargs:
            # Add the ticker as a new column
            self.macd_as_dataframe['ticker'] = self.kwargs['ticker']
            


    def process_all_stock_snapshot_results(self, all_stock_snapshot_results):
        if all_stock_snapshot_results is not None:
            self.ticker = [i.get('ticker', '') for i in all_stock_snapshot_results]
            self.todaysChangePerc = [float(i.get('todaysChangePerc', 0.0)) for i in all_stock_snapshot_results]
            self.todaysChange = [float(i.get('todaysChange', 0.0)) for i in all_stock_snapshot_results]
            self.updated = [i.get('updated', '') for i in all_stock_snapshot_results]
        else:
            # Handle the case where all_stock_snapshot_results is None
            self.ticker = []
            self.todaysChangePerc = []
            self.todaysChange = []
            self.updated = [] # Default to empty string if 'updated' is None


        self.day = [i.get('day') for i in all_stock_snapshot_results]
        self.day_open = [float(i.get('o')) for i in self.day]
        self.day_high = [float(i.get('h')) for i in self.day]
        self.day_low = [float(i.get('l')) for i in self.day]
        self.day_close = [float(i.get('c')) for i in self.day]
        self.day_volume = [float(i.get('v')) for i in self.day]
        self.day_vwap = [float(i.get('vw')) for i in self.day]




        self.lastQuote = [i.get('lastQuote') for i in all_stock_snapshot_results]
        self.ask  = [float(i.get('P')) for i in self.lastQuote]
        self.ask_size = [float(i.get('S')) for i in self.lastQuote]
        self.bid = [float(i.get('p')) for i in self.lastQuote]
        self.bid_size = [float(i.get('s')) for i in self.lastQuote]
        self.quote_timestamp = []
        for i in self.lastQuote:
            timestamp = i.get('t')
            if timestamp:
                try:
                    # Convert from nanoseconds to seconds
                    timestamp_seconds = int(timestamp) // 1_000_000_000
                    # Calculate remaining nanoseconds after converting to seconds
                    remaining_nanos = int(timestamp) % 1_000_000_000

                    # Create a datetime object for the Unix epoch
                    epoch = datetime(1970, 1, 1)
                    # Add the seconds and remaining nanoseconds to the epoch
                    converted_datetime = epoch + timedelta(seconds=timestamp_seconds, microseconds=remaining_nanos // 1000)

                    # Adjust for Eastern Time (ET) UTC-5
                    adjusted_datetime = converted_datetime - timedelta(hours=5)
                    self.quote_timestamp.append(adjusted_datetime.strftime('%Y-%m-%d %H:%M:%S'))
                except Exception as e:
                    self.quote_timestamp.append(f'Error: {e}')
            else:
                self.quote_timestamp.append('N/A')

        
        self.lastTrade = [i.get('lastTrade') for i in all_stock_snapshot_results]
        self.trade_conditions = [i.get('c') for i in self.lastTrade]
        self.trade_id = [i.get('i') for i in self.lastTrade]
        self.trade_price = [float(i.get('p')) for i in self.lastTrade]
        self.trade_size = [float(i.get('s')) for i in self.lastTrade]
        self.trade_timestamp = []
        for i in self.lastTrade:
            timestamp = i.get('t')
            if timestamp:
                try:
                    # Convert from nanoseconds to seconds
                    timestamp_seconds = int(timestamp) // 1_000_000_000
                    # Calculate remaining nanoseconds after converting to seconds
                    remaining_nanos = int(timestamp) % 1_000_000_000

                    # Create a datetime object for the Unix epoch
                    epoch = datetime(1970, 1, 1)
                    # Add the seconds and remaining nanoseconds to the epoch
                    converted_datetime = epoch + timedelta(seconds=timestamp_seconds, microseconds=remaining_nanos // 1000)

                    # Adjust for Eastern Time (ET) UTC-5
                    adjusted_datetime = converted_datetime - timedelta(hours=5)
                    self.trade_timestamp.append(adjusted_datetime.strftime('%Y-%m-%d %H:%M:%S'))
                except Exception as e:
                    self.trade_timestamp.append(f'Error: {e}')
            else:
                self.trade_timestamp.append('N/A')
        self.trade_exchange = [i.get('x') for i in self.lastTrade]


        self.min = [i.get('min') for i in all_stock_snapshot_results]
        self.min_accumulated_volume = [float(i.get('av')) if i.get('av') is not None else 'N/A' for i in all_stock_snapshot_results]
        self.min_timestamp = [
            (
                datetime.fromtimestamp(
                    float(i.get('t')) / 1000 if len(str(int(float(i.get('t'))))) == 13 else float(i.get('t')),
                    tz=timezone.utc
                ).astimezone(timezone(timedelta(hours=-5)))  # Eastern Time (ET) UTC-5
                .strftime('%Y-%m-%d %H:%M:%S') if i.get('t') is not None else 'N/A'
            )
            for i in self.min
        ]
        self.min_num_trades = [float(i.get('n')) if i.get('n') is not None else 'N/A' for i in self.min]
        self.min_open = [float(i.get('o')) if i.get('o') is not None else 'N/A' for i in self.min]
        self.min_high = [float(i.get('h')) if i.get('h') is not None else 'N/A' for i in self.min]
        self.min_low = [float(i.get('l')) if i.get('l') is not None else 'N/A' for i in self.min]
        self.min_close = [float(i.get('c')) if i.get('c') is not None else 'N/A' for i in self.min]
        self.min_vwap = [float(i.get('vw')) if i.get('vw') is not None else 'N/A' for i in self.min]


        self.prevDay = [i.get('prevDay') for i in all_stock_snapshot_results]
        self.prev_open = [float(i.get('o')) if i.get('o') is not None else 'N/A' for i in all_stock_snapshot_results]
        self.prev_high = [float(i.get('h')) if i.get('h') is not None else 'N/A' for i in self.prevDay]
        self.prev_low = [float(i.get('l')) if i.get('l') is not None else 'N/A' for i in self.prevDay]
        self.prev_close = [float(i.get('c')) if i.get('c') is not None else 'N/A' for i in self.prevDay]
        self.prev_volume = [float(i.get('v')) if i.get('v') is not None else 'N/A' for i in self.prevDay]
        self.prev_vwap = [float(i.get('vw')) if i.get('vw') is not None else 'N/A' for i in self.prevDay]


        self.all_stock_snapshot_data_dict = { 
            'ticker': self.ticker,
            'change_percent': self.todaysChangePerc,
            'change': self.todaysChange,
            'day_open': self.day_open,
            'day_high': self.day_high,
            'day_low': self.day_low,
            'day_close': self.day_close,
            'day_volume': self.day_volume,
            'day_vwap': self.day_vwap,
            'prev_open': self.prev_open,
            'prev_high': self.prev_high,
            'prev_low': self.prev_low,
            'prev_close': self.prev_close,
            'prev_volume': self.prev_volume,
            'prev_vwap': self.prev_vwap,
            'min_accumulated_volume': self.min_accumulated_volume,
            'min_open': self.min_open,
            'min_high': self.min_high,
            'min_low': self.min_low,
            'min_close': self.min_close,
            'min_vwap': self.min_vwap,
            'min_num_trades': self.min_num_trades,
            'min_timestamp': self.min_timestamp,
            'trade_conditions': self.trade_conditions,
            'trade_exchange': self.trade_exchange,
            'trade_price': self.trade_price,
            'trade_size': self.trade_size,
            'trade_id': self.trade_id,
            'trade_timestamp': self.trade_timestamp,
            'ask': self.ask,
            'ask_size': self.ask_size,
            'bid': self.bid,
            'bid_size': self.bid_size,
            'quote_timestamp': self.quote_timestamp
        }


        self.all_ticker_snapshots_as_dataframe = pd.DataFrame(self.all_stock_snapshot_data_dict)

    def process_aggregates_results(self, aggregates_results):
        self.agg_volume = [float(i.get('v')) if i.get('v') is not None else 'N/A' for i in aggregates_results]
        self.agg_vwap = [float(i.get('vw')) if i.get('vw') is not None else 'N/A' for i in aggregates_results]
        self.agg_open = [float(i.get('o')) if i.get('o') is not None else 'N/A' for i in aggregates_results]
        self.agg_close = [float(i.get('c')) if i.get('c') is not None else 'N/A' for i in aggregates_results]
        self.agg_high = [float(i.get('h')) if i.get('h') is not None else 'N/A' for i in aggregates_results]
        self.agg_low = [float(i.get('l')) if i.get('l') is not None else 'N/A' for i in aggregates_results]

        # Convert timestamps to Eastern Time

        eastern_timezone = timezone(timedelta(hours=-5))  # Eastern Time (ET) UTC-5
        self.agg_timestamp = [
            datetime.fromtimestamp(float(i.get('t')) / 1000, tz=timezone.utc).astimezone(eastern_timezone).strftime('%Y-%m-%d %H:%M:%S')
            if i.get('t') is not None
            else 'N/A'
            for i in aggregates_results
        ]
        self.agg_num_trades = [float(i.get('n')) if i.get('n') is not None else 'N/A' for i in aggregates_results]


        self.agg_data_dict = { 
            
            'agg_volume': self.agg_volume,
            'agg_vwap': self.agg_vwap,
            'agg_num_trades': self.agg_num_trades,
            'agg_open': self.agg_open,
            'agg_close': self.agg_close,
            'agg_high': self.agg_high,
            'agg_low': self.agg_low,
            'agg_timestamp': self.agg_timestamp,
            'agg_timespan': timespan
            
                                    }

        self.agg_as_dataframe = pd.DataFrame(self.agg_data_dict)

        if 'ticker' in self.kwargs:
            # Add the ticker as a new column
            self.agg_as_dataframe['ticker'] = self.kwargs['ticker']
            

    def process_ema_results(self, ema_results):
        self.underlying = ema_results.get('underlying')
        self.ema_values = ema_results.get('values')
        self.ema_timestamp = [
            (
                datetime.fromtimestamp(
                    float(i.get('timestamp')) / 1000 if len(str(int(float(i.get('timestamp'))))) == 13 else float(i.get('timestamp')),
                    tz=timezone.utc
                ).astimezone(timezone(timedelta(hours=-5)))  # Eastern Time (ET) UTC-5
                .strftime('%Y-%m-%d %H:%M:%S') if i.get('timestamp') is not None else 'N/A'
            )
            for i in self.ema_values
        ]

        self.ema_value = [float(i.get('value')) if i.get('value') is not None else 'N/A' for i in self.ema_values]

        self.ema_data_dict = { 
            'ema_timestamp': self.ema_timestamp,
            'ema_value': self.ema_value
        }
        self.ema_as_dataframe = pd.DataFrame(self.ema_data_dict)

        if 'ticker' in self.kwargs:
            # Add the ticker as a new column
            self.ema_as_dataframe['ticker'] = self.kwargs['ticker']
            

    def process_sma_results(self, sma_results):
        self.underlying = sma_results.get('underlying')
        self.sma_values = sma_results.get('values')
        self.sma_timestamp = [
            (
                datetime.fromtimestamp(
                    float(i.get('timestamp')) / 1000 if len(str(int(float(i.get('timestamp'))))) == 13 else float(i.get('timestamp')), 
                    tz=timezone.utc
                ).astimezone(timezone(timedelta(hours=-5)))  # Eastern Time (ET) UTC-5
                .strftime('%Y-%m-%d %H:%M:%S') if i.get('timestamp') is not None else 'N/A'
            )
            for i in self.sma_values
        ]

        self.sma_value = [float(i.get('value')) if i.get('value') is not None else 'N/A' for i in self.sma_values]


        self.sma_data_dict = { 
            'sma_value': self.sma_value,
            'sma_timestamp': self.sma_timestamp
        }

        self.sma_as_dataframe = pd.DataFrame(self.sma_data_dict)

        if 'ticker' in self.kwargs:
            # Add the ticker as a new column
            self.sma_as_dataframe['ticker'] = self.kwargs['ticker']

    # def process_single_snapshot_results(self, ticker_snapshot_results):
    #     self.ticker = ticker_snapshot_results['ticker']
    #     self.todaysChangePerc= float(ticker_snapshot_results['todaysChangePerc']) if ticker_snapshot_results['todaysChangePerc'] is not None else 'N/A'
    #     self.todaysChange= float(ticker_snapshot_results['todaysChange']) if ticker_snapshot_results['todaysChange'] is not None else 'N/A'
    #     self.updated= ticker_snapshot_results['updated']


    #     self.day= ticker_snapshot_results['day']
    #     self.day_open = float(self.day['o']) if self.day['o'] is not None else 'N/A'
    #     self.day_high = float(self.day['h']) if self.day['h'] is not None else 'N/A'
    #     self.day_low = float(self.day['l']) if self.day['l'] is not None else 'N/A'
    #     self.day_close = float(self.day['c']) if self.day['c'] is not None else 'N/A'
    #     self.day_volume = float(self.day['v']) if self.day['v'] is not None else 'N/A'
    #     self.day_vwap = float(self.day['v']) if self.day['v'] is not None else 'N/A'

    #     self.lastQuote= ticker_snapshot_results['lastQuote']
    #     self.ask = float(self.lastQuote['P']) if self.lastQuote['P'] is not None else 'N/A'
    #     self.ask_size = float(self.lastQuote['S']) if self.lastQuote['S'] is not None else 'N/A'
    #     self.bid = float(self.lastQuote['p']) if self.lastQuote['p'] is not None else 'N/A'
    #     self.bid_size = float(self.lastQuote['s']) if self.lastQuote['s'] is not None else 'N/A'
    #     timestamp = self.lastQuote.get('t')
    #     if timestamp:
    #         try:
    #             timestamp_seconds = int(timestamp) // 1_000_000_000
    #             remaining_nanos = int(timestamp) % 1_000_000_000

    #             # Create a datetime object for the Unix epoch
    #             epoch = datetime(1970, 1, 1)
    #             # Add the seconds and remaining nanoseconds to the epoch
    #             converted_datetime = epoch + timedelta(seconds=timestamp_seconds, microseconds=remaining_nanos // 1000)

    #             # Adjust for Eastern Time (ET) UTC-5
    #             adjusted_datetime = converted_datetime - timedelta(hours=5)
    #             self.quote_timestamp = adjusted_datetime.strftime('%Y-%m-%d %H:%M:%S')
    #         except Exception as e:
    #             self.quote_timestamp = f'Error: {e}'
    #     else:
    #         self.quote_timestamp = 'N/A'

    #     self.lastTrade= ticker_snapshot_results['lastTrade']
    #     self.trade_conditions = self.lastTrade['c']
    #     self.trade_id = self.lastTrade['i']
    #     self.trade_price = float(self.lastTrade['p']) if self.lastTrade['p'] is not None else 'N/A'
    #     self.trade_size = float(self.lastTrade['s']) if self.lastTrade['s'] is not None else 'N/A'
    #     timestamp = self.lastTrade.get('t')
    #     if timestamp:
    #         try:
    #             # Convert from nanoseconds to seconds
    #             timestamp_seconds = int(timestamp) // 1_000_000_000
    #             # Calculate remaining nanoseconds after converting to seconds
    #             remaining_nanos = int(timestamp) % 1_000_000_000

    #             # Create a datetime object for the Unix epoch
    #             epoch = datetime(1970, 1, 1)
    #             # Add the seconds and remaining nanoseconds to the epoch
    #             converted_datetime = epoch + timedelta(seconds=timestamp_seconds, microseconds=remaining_nanos // 1000)

    #             # Adjust for Eastern Time (ET) UTC-5
    #             adjusted_datetime = converted_datetime - timedelta(hours=5)
    #             self.trade_timestamp = adjusted_datetime.strftime('%Y-%m-%d %H:%M:%S')
    #         except Exception as e:
    #             self.trade_timestamp = f'Error: {e}'
    #     else:
    #         self.trade_timestamp = 'N/A'
    #     self.trade_exchange = float(self.lastTrade['x']) if self.lastTrade['x'] is not None else 'N/A'

    #     self.min= ticker_snapshot_results['min']
    #     self.min_accumulated_volume = float(self.min['av']) if self.min['av'] is not None else 'N/A'
    #     self.min_timestamp = []
    #     timestamp = self.min.get('t')
    #     if timestamp:
    #         try:
    #             # Convert from nanoseconds to seconds
    #             timestamp_seconds = int(timestamp) // 1_000_000_000
    #             # Calculate remaining nanoseconds after converting to seconds
    #             remaining_nanos = int(timestamp) % 1_000_000_000

    #             # Create a datetime object for the Unix epoch
    #             epoch = datetime(1970, 1, 1)
    #             # Add the seconds and remaining nanoseconds to the epoch
    #             converted_datetime = epoch + timedelta(seconds=timestamp_seconds, microseconds=remaining_nanos // 1000)

    #             # Adjust for Eastern Time (ET) UTC-5
    #             adjusted_datetime = converted_datetime - timedelta(hours=5)
    #             self.min_timestamp.append(adjusted_datetime.strftime('%Y-%m-%d %H:%M:%S'))
    #         except Exception as e:
    #             self.min_timestamp.append(f'Error: {e}')
    #     else:
    #         self.min_timestamp.append('N/A')

    #     self.min_num_trades = float(self.min['n']) if self.min['n'] is not None else 'N/A'
    #     self.min_open = float(self.min['o']) if self.min['o'] is not None else 'N/A'
    #     self.min_high = float(self.min['h']) if self.min['h'] is not None else 'N/A'
    #     self.min_low = float(self.min['l']) if self.min['l'] is not None else 'N/A'
    #     self.min_close = float(self.min['c']) if self.min['c'] is not None else 'N/A'
    #     self.min_vwap = float(self.min['vw']) if self.min['vw'] is not None else 'N/A'
                    

    #     self.prevDay= ticker_snapshot_results['prevDay']
    #     self.prev_open = float(self.prevDay['o']) if self.prevDay['o'] is not None else 'N/A'
    #     self.prev_high = float(self.prevDay['h']) if self.prevDay['h'] is not None else 'N/A'
    #     self.prev_low = float(self.prevDay['l']) if self.prevDay['l'] is not None else 'N/A'
    #     self.prev_close = float(self.prevDay['c']) if self.prevDay['c'] is not None else 'N/A'
    #     self.prev_volume = float(self.prevDay['v']) if self.prevDay['v'] is not None else 'N/A'
    #     self.prev_vwap = float(self.prevDay['vw']) if self.prevDay['vw'] is not None else 'N/A'


    #     self.single_ticker_snapshot_data_dict = { 
    #         'ticker': self.ticker,
    #         'change_percent': self.todaysChangePerc,
    #         'change': self.todaysChange,
    #         'day_open': self.day_open,
    #         'day_high': self.day_high,
    #         'day_low': self.day_low,
    #         'day_close': self.day_close,
    #         'day_volume': self.day_volume,
    #         'day_vwap': self.day_vwap,
    #         'prev_open': self.prev_open,
    #         'prev_high': self.prev_high,
    #         'prev_low': self.prev_low,
    #         'prev_close': self.prev_close,
    #         'prev_volume': self.prev_volume,
    #         'prev_vwap': self.prev_vwap,
    #         'min_accumulated_volume': self.min_accumulated_volume,
    #         'min_open': self.min_open,
    #         'min_high': self.min_high,
    #         'min_low': self.min_low,
    #         'min_close': self.min_close,
    #         'min_vwap': self.min_vwap,
    #         'min_num_trades': self.min_num_trades,
    #         'min_timestamp': self.min_timestamp,
    #         'trade_conditions': self.trade_conditions,
    #         'trade_exchange': self.trade_exchange,
    #         'trade_id': self.trade_id,
    #         'trade_price': self.trade_price,
    #         'trade_size': self.trade_size,
    #         'ask': self.ask,
    #         'ask_size': self.ask_size,
    #         'bid': self.bid,
    #         'bid_size': self.bid_size,
    #         'quote_timestamp': self.quote_timestamp
    #     }


    #     self.single_ticker_snapshot_as_dataframe = pd.DataFrame(self.single_ticker_snapshot_data_dict)


    def process_ticker_news_results(self, ticker_news_results):
        publisher = [i.get('publisher') for i in ticker_news_results]
        self.name = [i.get('name') for i in publisher]
        self.logo_url = [i.get('logo_url') for i in publisher]
        self.homepage_url = [i.get('homepage_url') for i in publisher]




        self.title = [i.get('title') for i in ticker_news_results]
        self.author = [i.get('author') for i in ticker_news_results]
        self.published_utc = [i.get('published_utc') for i in ticker_news_results]
        self.article_url = [i.get('article_url') for i in ticker_news_results]
        self.tickers = [i.get('tickers') for i in ticker_news_results]
        self.amp_url = [i.get('amp_url') for i in ticker_news_results]
        self.image_url = [i.get('image_url') for i in ticker_news_results]
        self.description = [i.get('description') for i in ticker_news_results]


        self.ticker_news_data_dict = { 
            'name': self.name,
            'logo_url': self.logo_url,
            'homepage_url': self.homepage_url,
            'title': self.title,
            'author': self.author,
            'article_url': self.article_url,
            'tickers': self.tickers,
            'amp_url': self.amp_url,
            'image_url': self.image_url,
            'description': self.description
        }


        self.ticker_news_as_dataframe = pd.DataFrame(self.ticker_news_data_dict)



    def process_annual_financial_results(self, financials_results):
        self.id = []
        self.sic = []
        self.start_date = []
        self.end_date = []

        self.id = [i.get('id') for i in financials_results]
        self.start_date = [i.get('start_date') for i in financials_results]
        self.end_date = [i.get('end_date') for i in financials_results]
        self.timeframe = [i.get('timeframe') for i in financials_results]
        self.fiscal_period = [i.get('fiscal_period') for i in financials_results]
        self.fiscal_year = [i.get('fiscal_year') for i in financials_results]
        self.cik = [i.get('cik') for i in financials_results]
        self.sic = [i.get('sic') for i in financials_results]
        self.tickers = [i.get('tickers') for i in financials_results]
        self.company_name = [i.get('company_name') for i in financials_results]
        self.financials = [i.get('financials') for i in financials_results]
        balance_sheet = [i.get('balance_sheet') for i in self.financials]
        comprehensive_income = [i.get('comprehensive_income') for i in self.financials]
        for i in comprehensive_income[0]:
            print(i)



        #income statement
        income_statement = [i.get('income_statement') for i in self.financials]
        # Replace None with 0.0 and convert to float for all lists
        net_income_loss = [i.get('net_income_loss', 0.0) for i in income_statement]
        income_loss_from_continuing_operations_before_tax = [i.get('income_loss_from_continuing_operations_before_tax') if i is not None else 0.0 for i in income_statement]
        basic_earnings_per_share = [i.get('basic_earnings_per_share') if i is not None else 0.0 for i in income_statement]
        net_income_loss_attributable_to_parent = [i.get('net_income_loss_attributable_to_parent') if i is not None else 0.0 for i in income_statement]
        revenues = [i.get('revenues') if i is not None else 0.0 for i in income_statement]
        net_income_loss_attributable_to_noncontrolling_interest = [i.get('net_income_loss_attributable_to_noncontrolling_interest') if i is not None else 0.0 for i in income_statement]
        net_income_loss_available_to_common_stockholders_basic = [i.get('net_income_loss_available_to_common_stockholders_basic') if i is not None else 0.0 for i in income_statement]
        gross_profit = [i.get('gross_profit') if i is not None else 0.0 for i in income_statement]
        operating_expenses = [i.get('operating_expenses') if i is not None else 0.0 for i in income_statement]
        preferred_stock_dividends_and_other_adjustments = [i.get('preferred_stock_dividends_and_other_adjustments') if i is not None else 0.0 for i in income_statement]
        cost_of_revenue = [i.get('cost_of_revenue') if i is not None else 0.0 for i in income_statement]
        costs_and_expenses = [i.get('costs_and_expenses') if i is not None else 0.0 for i in income_statement]
        basic_average_shares = [i.get('basic_average_shares') if i is not None else 0.0 for i in income_statement]
        interest_expense_operating = [i.get('interest_expense_operating') if i is not None else 0.0 for i in income_statement]
        income_loss_before_equity_method_investments = [i.get('income_loss_before_equity_method_investments') if i is not None else 0.0 for i in income_statement]
        nonoperating_income_loss = [i.get('nonoperating_income_loss') if i is not None else 0.0 for i in income_statement]
        benefits_costs_expenses = [i.get('benefits_costs_expenses') if i is not None else 0.0 for i in income_statement]
        income_loss_from_equity_method_investments = [i.get('income_loss_from_equity_method_investments') if i is not None else 0.0 for i in income_statement]
        income_tax_expense_benefit = [i.get('income_tax_expense_benefit') if i is not None else 0.0 for i in income_statement]
        income_tax_expense_benefit_deferred = [i.get('income_tax_expense_benefit_deferred') if i is not None else 0.0 for i in income_statement]
        diluted_earnings_per_share = [i.get('diluted_earnings_per_share') if i is not None else 0.0 for i in income_statement]
        operating_income_loss = [i.get('operating_income_loss') if i is not None else 0.0 for i in income_statement]
        income_loss_from_continuing_operations_after_tax = [i.get('income_loss_from_continuing_operations_after_tax') if i is not None else 0.0 for i in income_statement]
        diluted_average_shares = [i.get('diluted_average_shares') if i is not None else 0.0 for i in income_statement]



        #balance sheet
        equity_attributable_to_parent = [i.get('equity_attributable_to_parent') if i else None for i in balance_sheet]
        assets = [i.get('assets') if i else None for i in balance_sheet]
        noncurrent_liabilities = [i.get('noncurrent_liabilities') if i else None for i in balance_sheet]
        other_noncurrent_assets = [i.get('other_noncurrent_assets') if i else None for i in balance_sheet]
        inventory = [i.get('inventory') if i else None for i in balance_sheet]
        equity = [i.get('equity') if i else None for i in balance_sheet]
        accounts_payable = [i.get('accounts_payable') if i else None for i in balance_sheet]
        long_term_debt = [i.get('long_term_debt') if i else None for i in balance_sheet]
        equity_attributable_to_noncontrolling_interest = [i.get('equity_attributable_to_noncontrolling_interest') if i else None for i in balance_sheet]
        fixed_assets = [i.get('fixed_assets') if i else None for i in balance_sheet]
        liabilities = [i.get('liabilities') if i else None for i in balance_sheet]
        noncurrent_assets = [i.get('noncurrent_assets') if i else None for i in balance_sheet]
        current_liabilities = [i.get('current_liabilities') if i else None for i in balance_sheet]
        other_current_liabilities = [i.get('other_current_liabilities') if i else None for i in balance_sheet]
        other_current_assets = [i.get('other_current_assets') if i else None for i in balance_sheet]
        current_assets = [i.get('current_assets') if i else None for i in balance_sheet]
        liabilities_and_equity = [i.get('liabilities_and_equity') if i else None for i in balance_sheet]
        other_noncurrent_liabilities = [i.get('other_noncurrent_liabilities') if i else None for i in balance_sheet]


        #comprehensive income
        comprehensive_income_loss_attributable_to_parent= [i.get('comprehensive_income_loss_attributable_to_parent') if i else None for i in balance_sheet]
        comprehensive_income_loss_attributable_to_noncontrolling_interest= [i.get('comprehensive_income_loss_attributable_to_noncontrolling_interest') if i else None for i in balance_sheet]
        comprehensive_income_loss= [i.get('comprehensive_income_loss') if i else None for i in balance_sheet]
        other_comprehensive_income_loss= [i.get('other_comprehensive_income_loss') if i else None for i in balance_sheet]


        #cashflow statement
        cash_flow_statement = [i.get('cash_flow_statement') for i in self.financials]
        net_cash_flow_from_financing_activities = [i.get('net_cash_flow_from_financing_activities', 0.0) for i in cash_flow_statement]
        net_cash_flow_from_investing_activities_continuing = [i.get('net_cash_flow_from_investing_activities_continuing', 0.0) for i in cash_flow_statement]
        net_cash_flow_continuing = [i.get('net_cash_flow_continuing', 0.0) for i in cash_flow_statement]
        net_cash_flow_from_operating_activities = [i.get('net_cash_flow_from_operating_activities', 0.0) for i in cash_flow_statement]
        net_cash_flow_from_investing_activities = [i.get('net_cash_flow_from_investing_activities', 0.0) for i in cash_flow_statement]
        net_cash_flow_from_financing_activities_continuing = [i.get('net_cash_flow_from_financing_activities_continuing', 0.0) for i in cash_flow_statement]
        net_cash_flow_from_operating_activities_continuing = [i.get('net_cash_flow_from_operating_activities_continuing', 0.0) for i in cash_flow_statement]
        net_cash_flow = [i.get('net_cash_flow', 0.0) for i in cash_flow_statement]


        
        self.value = [
            item.get('value') if isinstance(item, dict) else None
            for sublist in [
                equity_attributable_to_parent, assets, noncurrent_liabilities, other_current_assets, inventory, equity, accounts_payable, long_term_debt, equity_attributable_to_noncontrolling_interest, fixed_assets, liabilities, noncurrent_assets, current_liabilities, other_current_liabilities, other_current_assets, current_assets, liabilities_and_equity, other_noncurrent_liabilities, other_noncurrent_assets,
                # Add the new attributes here
                income_loss_from_continuing_operations_before_tax, basic_earnings_per_share, net_income_loss_attributable_to_parent, revenues, net_income_loss_attributable_to_noncontrolling_interest, net_income_loss_available_to_common_stockholders_basic, gross_profit, operating_expenses, preferred_stock_dividends_and_other_adjustments, cost_of_revenue, costs_and_expenses, basic_average_shares, interest_expense_operating, income_loss_before_equity_method_investments, nonoperating_income_loss, benefits_costs_expenses, income_loss_from_equity_method_investments, income_tax_expense_benefit, income_tax_expense_benefit_deferred, diluted_earnings_per_share, operating_income_loss, income_loss_from_continuing_operations_after_tax, diluted_average_shares, net_income_loss,
                #comp income
                comprehensive_income_loss_attributable_to_parent, comprehensive_income_loss_attributable_to_noncontrolling_interest, comprehensive_income_loss, other_comprehensive_income_loss,
                #netcashflow
                net_cash_flow_from_financing_activities, net_cash_flow_continuing, net_cash_flow_from_investing_activities_continuing, net_cash_flow_from_operating_activities, net_cash_flow_from_investing_activities, net_cash_flow_from_investing_activities_continuing, net_cash_flow_from_financing_activities_continuing, net_cash_flow_from_operating_activities_continuing, net_cash_flow
            ]
            for item in sublist
        ]
        self.unit = [
            item.get('unit') if isinstance(item, dict) else None
            for sublist in [
                equity_attributable_to_parent, assets, noncurrent_liabilities, other_current_assets, inventory, equity, accounts_payable, long_term_debt, equity_attributable_to_noncontrolling_interest, fixed_assets, liabilities, noncurrent_assets, current_liabilities, other_current_liabilities, other_current_assets, current_assets, liabilities_and_equity, other_noncurrent_liabilities, other_noncurrent_assets,
                # Add the new attributes here
                income_loss_from_continuing_operations_before_tax, basic_earnings_per_share, net_income_loss_attributable_to_parent, revenues, net_income_loss_attributable_to_noncontrolling_interest, net_income_loss_available_to_common_stockholders_basic, gross_profit, operating_expenses, preferred_stock_dividends_and_other_adjustments, cost_of_revenue, costs_and_expenses, basic_average_shares, interest_expense_operating, income_loss_before_equity_method_investments, nonoperating_income_loss, benefits_costs_expenses, income_loss_from_equity_method_investments, income_tax_expense_benefit, income_tax_expense_benefit_deferred, diluted_earnings_per_share, operating_income_loss, income_loss_from_continuing_operations_after_tax, diluted_average_shares, net_income_loss,
                #comp income
                comprehensive_income_loss_attributable_to_parent, comprehensive_income_loss_attributable_to_noncontrolling_interest, comprehensive_income_loss, other_comprehensive_income_loss,
                #netcashflow
                net_cash_flow_from_financing_activities, net_cash_flow_continuing, net_cash_flow_from_investing_activities_continuing, net_cash_flow_from_operating_activities, net_cash_flow_from_investing_activities, net_cash_flow_from_investing_activities_continuing, net_cash_flow_from_financing_activities_continuing, net_cash_flow_from_operating_activities_continuing, net_cash_flow
            ]
            for item in sublist
        ]

        self.label = [
            item.get('label') if isinstance(item, dict) else None
            for sublist in [
                equity_attributable_to_parent, assets, noncurrent_liabilities, other_current_assets, inventory, equity, accounts_payable, long_term_debt, equity_attributable_to_noncontrolling_interest, fixed_assets, liabilities, noncurrent_assets, current_liabilities, other_current_liabilities, other_current_assets, current_assets, liabilities_and_equity, other_noncurrent_liabilities, other_noncurrent_assets,
                # Add the new attributes here
                income_loss_from_continuing_operations_before_tax, basic_earnings_per_share, net_income_loss_attributable_to_parent, revenues, net_income_loss_attributable_to_noncontrolling_interest, net_income_loss_available_to_common_stockholders_basic, gross_profit, operating_expenses, preferred_stock_dividends_and_other_adjustments, cost_of_revenue, costs_and_expenses, basic_average_shares, interest_expense_operating, income_loss_before_equity_method_investments, nonoperating_income_loss, benefits_costs_expenses, income_loss_from_equity_method_investments, income_tax_expense_benefit, income_tax_expense_benefit_deferred, diluted_earnings_per_share, operating_income_loss, income_loss_from_continuing_operations_after_tax, diluted_average_shares, net_income_loss,
                #comp income
                comprehensive_income_loss_attributable_to_parent, comprehensive_income_loss_attributable_to_noncontrolling_interest, comprehensive_income_loss, other_comprehensive_income_loss,
                #netcashflow
                net_cash_flow_from_financing_activities, net_cash_flow_continuing, net_cash_flow_from_investing_activities_continuing, net_cash_flow_from_operating_activities, net_cash_flow_from_investing_activities, net_cash_flow_from_investing_activities_continuing, net_cash_flow_from_financing_activities_continuing, net_cash_flow_from_operating_activities_continuing, net_cash_flow
            ]
            for item in sublist
        ]

        self.order = [
            item.get('order') if isinstance(item, dict) else None
            for sublist in [
                equity_attributable_to_parent, assets, noncurrent_liabilities, other_current_assets, inventory, equity, accounts_payable, long_term_debt, equity_attributable_to_noncontrolling_interest, fixed_assets, liabilities, noncurrent_assets, current_liabilities, other_current_liabilities, other_current_assets, current_assets, liabilities_and_equity, other_noncurrent_liabilities, other_noncurrent_assets,
                # Add the new attributes here
                income_loss_from_continuing_operations_before_tax, basic_earnings_per_share, net_income_loss_attributable_to_parent, revenues, net_income_loss_attributable_to_noncontrolling_interest, net_income_loss_available_to_common_stockholders_basic, gross_profit, operating_expenses, preferred_stock_dividends_and_other_adjustments, cost_of_revenue, costs_and_expenses, basic_average_shares, interest_expense_operating, income_loss_before_equity_method_investments, nonoperating_income_loss, benefits_costs_expenses, income_loss_from_equity_method_investments, income_tax_expense_benefit, income_tax_expense_benefit_deferred, diluted_earnings_per_share, operating_income_loss, income_loss_from_continuing_operations_after_tax, diluted_average_shares, net_income_loss,
                #comp income
                comprehensive_income_loss_attributable_to_parent, comprehensive_income_loss_attributable_to_noncontrolling_interest, comprehensive_income_loss, other_comprehensive_income_loss,
                #netcashflow
                net_cash_flow_from_financing_activities, net_cash_flow_continuing, net_cash_flow_from_investing_activities_continuing, net_cash_flow_from_operating_activities, net_cash_flow_from_investing_activities, net_cash_flow_from_investing_activities_continuing, net_cash_flow_from_financing_activities_continuing, net_cash_flow_from_operating_activities_continuing, net_cash_flow
            ]
            for item in sublist
        ]

        fiscal_years = []
        fiscal_periods = []
        start_dates = []
        end_dates = []
        timeframes = []

        
        for i in range(len(self.label)):
            fiscal_year = self.fiscal_year[i % 10]  # Use modulo to cycle through the 10 fiscal years
            fiscal_period = self.fiscal_period[i % 10]
            end_date = self.end_date[i % 10]
            start_date = self.start_date[i % 10]
            timeframe = self.timeframe[i % 10]

            fiscal_years.append(fiscal_year)
            fiscal_periods.append(fiscal_period)
            start_dates.append(start_date)
            end_dates.append(end_date)
            timeframes.append(timeframe)

        print(len(fiscal_years), len(fiscal_periods), len(start_dates), len(end_dates))
        self.financials_data_dict = { 
            'fiscal_year': fiscal_years,
            'fiscal_period': fiscal_periods,
            'start_date': start_dates,
            'end_date': end_dates,
            'metric': self.label,
            'value': self.value,
            'unit': self.unit

        }


        self.as_financials_df = pd.DataFrame(self.financials_data_dict)
        self.as_financials_df = self.format_large_numbers_in_dataframe(self.as_financials_df).sort_values('end_date', ascending=False)
        if 'ticker' in self.kwargs:
            # Add the ticker as a new column
            self.as_financials_df['ticker'] = self.kwargs['ticker']
            



    def convert_timestamp(self, timestamp, from_milliseconds=True):
        try:
            if from_milliseconds:
                timestamp = float(timestamp) / 1000
            dt_object = datetime.fromtimestamp(timestamp, tz=timezone.utc)
            return dt_object.astimezone(timezone(timedelta(hours=-5))).strftime('%Y-%m-%d %H:%M:%S')
        except Exception as e:
            return f'Error: {e}'
        
    def make_polygon_request(self, endpoint, params):
        base_url = "https://api.polygon.io"
        api_key = os.environ.get('YOUR_POLYGON_KEY')
        full_url = f"{base_url}{endpoint}?apiKey={api_key}"
        for key, value in params.items():
            full_url += f"&{key}={value}"
        response = requests.get(full_url)
        return response.json().get('results')
        

    def get_rsi(self, ticker, timespan, limit):
        endpoint = f"/v1/indicators/rsi/{ticker}"
        params = {"timespan": timespan, "adjusted": "true", "window": 14, "order": "desc", "limit": limit}
        rsi_response = self.make_polygon_request(endpoint, params)
        return PolygonMaster(rsi_results=rsi_response, ticker=ticker)
    
    def get_macd(self, ticker, timespan, limit):
        endpoint = f"/v1/indicators/macd/{ticker}"
        params = {"timespan": timespan, "adjusted": "true", "series_type": "close", "order": "desc", "limit": limit}
        sma_response = self.make_polygon_request(endpoint, params)
        return PolygonMaster(macd_results=sma_response, ticker=ticker)
    
    def get_sma(self, ticker, timespan, window, limit):
        endpoint = f"/v1/indicators/sma/{ticker}"
        params = {"timespan": timespan, "adjusted": "true", "window": window, "series_type": "close", "order": "desc", "limit": limit}
        sma_response = self.make_polygon_request(endpoint, params)
        return PolygonMaster(sma_results=sma_response, ticker=ticker)

    def get_ema(self, ticker, timespan, window, limit):
        endpoint = f"/v1/indicators/ema/{ticker}"
        params = {"timespan": timespan, "adjusted": "true", "window": window, "series_type": "close", "order": "desc", "limit": limit}
        ema_response = self.make_polygon_request(endpoint, params)
        return PolygonMaster(ema_results=ema_response, ticker=ticker)

    def get_all_tickers_snapshot(self):
        endpoint = "/v2/snapshot/locale/us/markets/stocks/tickers"
        all_tickers_snapshot_response = self.make_polygon_request(endpoint, {})
        return PolygonMaster(all_stock_snapshot_results=all_tickers_snapshot_response)

    def get_aggregates(self,ticker, multiplier, timespan, date_from, date_to, limit):
        endpoint = f"/v2/aggs/ticker/{ticker}/range/{multiplier}/{timespan}/{date_from}/{date_to}"
        params = {"adjusted": "true", "sort": "asc", "limit": limit}
        aggregates_response = self.make_polygon_request(endpoint, params)
        return PolygonMaster(aggregates_results=aggregates_response, ticker=ticker)

    def get_ticker_news(self, ticker):
        endpoint = f"/v2/reference/news"
        params = {"ticker": ticker}
        ticker_news_response = self.make_polygon_request(endpoint, params)
        return PolygonMaster(ticker_news_results=ticker_news_response, ticker=ticker)

    def get_financials(self, ticker, timeframe):
        endpoint = f"/vX/reference/financials"
        params = {"ticker": ticker, "timeframe": timeframe}
        financials_response = self.make_polygon_request(endpoint, params)
        return PolygonMaster(financials_results=financials_response, ticker=ticker)
    

master = PolygonMaster()
opts = PolygonOptions(user='chuck', database='poly')
async def main():
    ticker='AAPL'
    timespan='day'
    multiplier=1
    date_to = '2023-12-31'
    date_from = '2023-12-01'
    limit=100
    rsi = master.get_rsi(ticker=ticker,timespan=timespan, limit=limit)
    
    aggs = master.get_aggregates(ticker,multiplier,timespan,date_from,date_to,limit)
    financials = master.get_financials(ticker,timeframe='quarterly')
    ema = master.get_ema(ticker,timespan,window,limit)
    sma = master.get_sma(ticker,timespan,window,limit)
    macd = master.get_macd(ticker,timespan,limit)
    news = master.get_ticker_news(ticker)
 

    await opts.connect()

    await opts.batch_insert_dataframe(rsi.rsi_as_dataframe,table_name='rsi', unique_columns='rsi_timespan, rsi_timestamp, ticker')
    await opts.batch_insert_dataframe(aggs.agg_as_dataframe, table_name='aggs', unique_columns='agg_timespan, agg_timestamp, ticker')
    await opts.batch_insert_dataframe(financials.as_financials_df, table_name='financials', unique_columns='ticker')
    await opts.batch_insert_dataframe(ema.ema_as_dataframe, table_name='ema', unique_columns='ticker')
    await opts.batch_insert_dataframe(sma.sma_as_dataframe, table_name='sma', unique_columns='ticker')
    await opts.batch_insert_dataframe(macd.macd_as_dataframe, table_name='macd', unique_columns='ticker')
    await opts.batch_insert_dataframe(news.ticker_news_as_dataframe, table_name='news', unique_columns='insertion_timestamp')
    #await opts.batch_insert_dataframe(snapshots.all_ticker_snapshots_as_dataframe, table_name='snapshot', unique_columns='insertion_timestamp')


asyncio.run(main())


import aiohttp
import asyncio
import asyncpg
import json
from typing import Optional

class CFRSearchSummaryAPI:
    def __init__(self, base_url: str, db_connection_string: Optional[str] = None):
        self.base_url = base_url
        self.db_connection_string = db_connection_string
        self.session = aiohttp.ClientSession()
        self.db_pool = None

    async def connect_to_db(self):
        if self.db_connection_string:
            self.db_pool = await asyncpg.create_pool(dsn=self.db_connection_string)

    async def create_table(self):
        async with self.db_pool.acquire() as connection:
            await connection.execute('''
                CREATE TABLE IF NOT EXISTS search_summary (
                    id SERIAL PRIMARY KEY,
                    query TEXT,
                    date DATE,
                    last_modified_after DATE,
                    last_modified_on_or_after DATE,
                    last_modified_before DATE,
                    last_modified_on_or_before DATE,
                    response JSONB,
                    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
                    UNIQUE (query, date, last_modified_after, last_modified_on_or_after, last_modified_before, last_modified_on_or_before)
                )
            ''')

    async def insert_data(self, data):
        # Flatten the response by converting it to a JSON string
        response_json = json.dumps(data[-1])
        async with self.db_pool.acquire() as connection:
            await connection.execute('''
                INSERT INTO search_summary (
                    query, date, last_modified_after, last_modified_on_or_after, last_modified_before, last_modified_on_or_before, response
                ) VALUES ($1, $2, $3, $4, $5, $6, $7)
                ON CONFLICT (query, date, last_modified_after, last_modified_on_or_after, last_modified_before, last_modified_on_or_before)
                DO NOTHING
            ''', *(data[:-1] + (response_json,)))

    async def fetch_results(self, query):
        async with self.session.get(f"https://www.ecfr.gov/api/search/v1/results?query=options&per_page=20&page=1&order=relevance") as response:
            print(f"https://www.ecfr.gov/api/api/search/v1/results?query={query}")
            data = await response.json()
            return data     



    async def fetch_summary(self, query):
        #params = {k: v for k, v in params.items() if v is not None}
        async with self.session.get(f"https://www.ecfr.gov/api/search/v1/summary?query={query}") as response:
            data = await response.json()
            return data

    async def fetch_and_store_summary(self, **params):
        data = await self.fetch_summary(**params)
        if self.db_pool:
            await self.insert_data((
                params.get('query'),
                params.get('date'),
                params.get('last_modified_after'),
                params.get('last_modified_on_or_after'),
                params.get('last_modified_before'),
                params.get('last_modified_on_or_before'),
                data
            ))
        return data

    async def close(self):
        await self.session.close()
        if self.db_pool:
            await self.db_pool.close()

async def main():
    # Example usage
    base_url = 'https://example.com'
    db_connection_string = 'postgresql://postgres:fud@localhost:5432/cfr'

    api = CFRSearchSummaryAPI(base_url, db_connection_string)
    # await api.connect_to_db()
    # await api.create_table()

    # Fetch and store summary
    summary_data = await api.fetch_results(query='climate change')
    print(summary_data)

    await api.close()

if __name__ == '__main__':
    asyncio.run(main())


import asyncio
import os
from dotenv import load_dotenv
load_dotenv()
from discord_webhook import DiscordEmbed, AsyncDiscordWebhook


class WebhookFeeds:
    def __init__(self):
        self.embed = DiscordEmbed()



    async def million_dollar_trades(message, conditions, exchange, dollar_cost):

        """Alerts discord when a trade size of $1,000,000 or more is captured via 
        EquityTrade Feed
        
        
        """





        if dollar_cost >= 1000000:
            print(message)
            hook = AsyncDiscordWebhook(os.environ.get('MILLION_DOLLAR_TRADE'), content=f"> $1,000,000 + trade | {message.symbol} | Price: ${message.price} | Size: {message.size} | Time: {timestamp}")


            embed = DiscordEmbed(title=f">> $1,000,000 Dollar Trade - {message.symbol}", description=f"# > **{message.symbol}** just traded for a dollar value of ${format_large_number(dollar_cost)}", content=f"> # $100k+ trade | {message.symbol} | Price: ${message.price} | Size: {message.size}\n# > Exchange: **{message.exchange}**")
            embed.add_embed_field(name=f"Price & Size:", value=f"# > ${message.price}\n# > {message.size}")
            embed.add_embed_field(name=f"Conditions:", value=f"# > **{conditions}**")
            embed.add_embed_field(name=f"Exchange:", value=f"# > **{exchange}")




            await hook.execute()


import os
from dotenv import load_dotenv
load_dotenv()
from apis.polygonio.polygon_options import PolygonOptions
from apis.webull.webull_options import WebullOptions
import pandas as pd
wo = WebullOptions(connection_string=os.environ.get('WEBULL_OPTIONS'))
opts = PolygonOptions()


selected_ticker = None
async def ticker_autocomp(inter, ticker: str):
    global selected_ticker
    await opts.connect()
    if not ticker:
        return ["TYPE A TICKER"]

    # assuming db_manager is globally defined and connected
    query = f"""SELECT DISTINCT ticker FROM options_data WHERE ticker LIKE '{ticker}%' LIMIT 24;"""
    results = await opts.fetch(query)
    
    if not results:
        return []

    # convert to DataFrame just for demonstration, not actually needed
    df = pd.DataFrame(results, columns=['ticker'], index=None)
    selected_ticker = ticker
    # Return the symbols
    return df['ticker'].str.upper().tolist()[:24]


async def strike_autocomp(inter, strike: str):
    global selected_ticker  # Declare the variable as global to read it
    if not strike:
        return ["TYPE A STRIKE"]
        
    query = f"""SELECT DISTINCT CAST(strike AS text) FROM options_data WHERE ticker = '{selected_ticker}' AND CAST(strike AS text) LIKE '{strike}%' LIMIT 24;"""
    results = await opts.fetch(query)
    if not results:
        return []
    df = pd.DataFrame(results, columns=['strike'])
    # Return the symbols
    return df['strike'].str.lower().tolist()[:24]

    


async def expiry_autocomp(inter, expiry: str):

    global selected_ticker  # Declare the variable as global to read it
    if not expiry:
        return ["CHOOSE", "AN", "EXPIRY"]
        
    query = f"""SELECT DISTINCT CAST(expiry AS text) FROM options_data WHERE ticker = '{selected_ticker}' AND CAST(expiry AS text) LIKE '{expiry}%' LIMIT 24;"""
    results = await opts.fetch(query)
    if not results:
        return []
    df = pd.DataFrame(results, columns=['expiry'])
    # Return the symbols
    df['expiry'] = pd.to_datetime(df['expiry'])
    df['expiry'] = df['expiry'].apply(lambda x: x.strftime('%y%m%d'))
    return df['expiry'].str.lower().tolist()[:24]





from dataclasses import dataclass, asdict
from datetime import datetime

import pandas as pd

@dataclass
class JimCramerData:
    ticker: str
    date: datetime
    segment: str
    call: str
    price: float

    def as_dataframe(self) -> pd.DataFrame:
        """Converts the object to a pandas DataFrame."""
        return pd.DataFrame([asdict(self)])
    
    def insert_into_pg(self, conn):
        cursor = conn.cursor()
        insert_query = """INSERT INTO jim_cramer (ticker, date, segment, call, price) VALUES (%s, %s, %s, %s, %s);"""
        cursor.execute(insert_query, (self.ticker, self.date, self.segment, self.call, self.price))
        conn.commit()



from dataclasses import dataclass, asdict
from datetime import datetime

import pandas as pd

@dataclass
class ReverseRepoData:
    date: datetime
    amount: float
    num_parties: int
    average: float
    moving_avg: float


    def as_dataframe(self) -> pd.DataFrame:
        """Converts the object to a pandas DataFrame."""
        return pd.DataFrame([asdict(self)])

    def insert_into_pg(self, conn):
        cursor = conn.cursor()
        insert_query = """INSERT INTO reverse_repo (date, amount, num_parties, average, moving_avg) VALUES (%s, %s, %s, %s, %s);"""
        cursor.execute(insert_query, (self.date, self.amount, self.num_parties, self.average, self.moving_avg))
        conn.commit()




from dotenv import load_dotenv
load_dotenv()
import sys
from pathlib import Path
# Add the project directory to the sys.path
project_dir = str(Path(__file__).resolve().parents[1])
if project_dir not in sys.path:
    sys.path.append(project_dir)

from fudstop.apis.polygonio.polygon_options import PolygonOptions
from list_sets.ticker_lists import most_active_tickers
import asyncio


db = PolygonOptions(user='chuck', database='markets')
async def main(ticker):
    
    all_options = await db.get_option_chain_all(underlying_asset=ticker)

    asyncio.create_task(db.batch_insert_dataframe(all_options.df,table_name='poly_opts', unique_columns='option_symbol'))
async def run_main():
    await db.connect()
    tasks = [main(i) for i in most_active_tickers]


    await asyncio.gather(*tasks)



asyncio.run(run_main())


import sys
from pathlib import Path

# Add the project directory to the sys.path
project_dir = str(Path(__file__).resolve().parents[2])
if project_dir not in sys.path:
    sys.path.append(project_dir)
import os
from dotenv import load_dotenv
load_dotenv()

import pandas as pd
import asyncio

from apis.polygonio.polygon_options import PolygonOptions
from apis.polygonio.async_polygon_sdk import Polygon
from apis.helpers import convert_datetime_list, get_human_readable_string
opts = PolygonOptions()
poly = Polygon(os.environ.get('POLYGON_STRING'))



async def process_ticker(opts, ticker):
    try:
        print(ticker)
        components = get_human_readable_string(ticker)
        underlying_symbol = components.get('underlying_symbol')
        strike = components.get('strike_price')
        call_put = components.get('call_put')
        expiry_date = components.get('expiry_date')

        aggs = await opts.option_aggregates(ticker, timespan='second', as_dataframe=True)
        aggs = aggs.rename(columns={'v': 'volume', 'vw': 'vwap', 'o': 'open', 'h': 'high', 'l': 'low', 't': 'timestamp', 'n': 'trades'})
        aggs['timestamp'] = pd.to_datetime(aggs['timestamp'], unit='ms').dt.tz_localize('UTC').dt.tz_convert('US/Eastern')
        aggs['timestamp'] = aggs['timestamp'].dt.strftime('%Y-%m-%d %H:%M:%S')
        aggs['option_symbol'] = ticker
        aggs['ticker'] = underlying_symbol
        aggs['strike'] = strike
        aggs['call_put'] = str(call_put).lower()
        aggs['expiry'] = expiry_date

        # Insert data into the database immediately after processing
        await opts.batch_insert_dataframe(aggs, table_name='option_aggs', unique_columns='option_symbol, timestamp')
    except Exception as e:
        print(f"Error processing ticker {ticker}: {e}")


async def process_batch(opts, tickers):
    for ticker in tickers:
        await process_ticker(opts, ticker)

async def get_all_aggregates():
    
       
    tickers = await opts.get_tickers()

    batch_size = 250  # Process 250 tickers at a time
    tasks = []
    for i in range(0, len(tickers), batch_size):
        batch = tickers[i:i + batch_size]
        # Create an asynchronous task for each batch
        task = asyncio.create_task(process_batch(opts, batch))
        tasks.append(task)

    # Run all batch processing tasks concurrently
    await asyncio.gather(*tasks)

asyncio.run(get_all_aggregates())


import disnake

from disnake.ext import commands

import requests
import pandas as pd
from apis.helpers import format_large_number
from discord_.bot_menus.pagination import AlertMenus, PageSelect
class IBRS(commands.Cog):

    def __init__(self, bot):
        self.bot=bot





    @commands.slash_command()
    async def ibrd(self, inter):
        pass





    @ibrd.sub_command()
    async def flows_commitments(self, inter:disnake.AppCmdInter, year:str='2023'):
        """loans and IDA Credits are made to, or guaranteed by, countries that are members of IBRD and IDA"""
        await inter.response.defer()
        r = requests.get(f"https://finances.worldbank.org/resource/7ipw-i7ht.json?fiscal_year={year}").json()

        df = pd.DataFrame(r)

        embeds=[]
        for i,row in df.iterrows():
            financier = row['financier']
            year = row['fiscal_year']
            region = row['region']
            country = row['country']
            gross_disbursement = format_large_number(row['gross_disbursement'])
            repayments = format_large_number(row['repayments'])
            net_disbursement = format_large_number(row['net_disbursement'])
            interest = format_large_number(row['interest'])
            fees = format_large_number(row['fees'])
            ibrd_commitments_us = format_large_number(row['ibrd_commitments_us'])
            ida_non_concessional = format_large_number(row['ida_non_concessional'])
            ida_concessional_commitments = format_large_number(row['ida_concessional_commitments'])
            ida_other_commitments_us = format_large_number(row['ida_other_commitments_us'])

            embed = disnake.Embed(title=f"IBRD and IDA Net Flows & Commitments", description=f"""```py\nThe International Bank for Reconstruction and Development (IBRD) loans and International Development Association (IDA) credits are public and publicly guaranteed debt extended by the World Bank Group. IBRD loans and IDA Credits are made to, or guaranteed by, countries that are members of IBRD and IDA.
IBRD lends at market rates.\n\nIDA provides development credits, grants, and guarantees to its recipient member countries at concessional rates. IDA also has Non-Concessional Lending which is priced at market rates (similar to IBRD). IBRD and IDA net flows and commitments dataset contains IBRD and IDA commitments, gross disbursements, repayments, net disbursements (disbursements net of repayments), Interest charges, and fees (commitment fee, front end fee, service charges, and guarantee fees) at a country level. Data are in U.S. dollars calculated using historical rates.```""", color=disnake.Colour.dark_green())
            embed.add_field(name=f"Financier:", value=f"> **{financier}**")
            embed.add_field(name=f"Year:", value=f"> **{year}**")
            embed.add_field(name=f"Region:", value=f"> **{region}**")
            embed.add_field(name=f"Country:", value=f"> **{country}**")
            embed.add_field(name=f"Disbursement:", value=f"> Gross: **${gross_disbursement}**\n> Net: **${net_disbursement}**")
            embed.add_field(name=f"Repayments:", value=f"> **${repayments}**")
            embed.add_field(name=f"Interest & Fees:", value=f"> **{interest}** // **{fees}**")
            embed.add_field(name=f"U.S. Commitments:", value=f"> **${ibrd_commitments_us}**")
            embed.add_field(name=f"Concessional vs Non IDA:", value=f"> Concessional: **${ida_concessional_commitments}**\n> Non: **${ida_non_concessional}**")
            embed.add_field(name=f"Other Commitments - U.S.:", value=f"> **{ida_other_commitments_us}**")
            embeds.append(embed)


        await inter.edit_original_message(embed=embeds[0], view=AlertMenus(embeds))




def setup(bot: commands.Bot):
    bot.add_cog(IBRS(bot))
    print('IBRS READY!')


import sys
from pathlib import Path

# Add the project directory to the sys.path
project_dir = str(Path(__file__).resolve().parents[1])
if project_dir not in sys.path:
    sys.path.append(project_dir)
import os
from dotenv import load_dotenv
load_dotenv()
from async_polygon_sdk import Polygon



import asyncio
from datetime import datetime, timedelta
class RSIScanner(Polygon):
    def __init__(self, connection_string):
        self.polygon = Polygon(connection_string)
        self.connection_string = connection_string
        self.api_key = os.environ.get('YOUR_POLYGON_KEY')
        self.today = datetime.now().strftime('%Y-%m-%d')
        self.yesterday = (datetime.now() - timedelta(days=1)).strftime('%Y-%m-%d')
        self.tomorrow = (datetime.now() + timedelta(days=1)).strftime('%Y-%m-%d')
        self.thirty_days_ago = (datetime.now() - timedelta(days=30)).strftime('%Y-%m-%d')
        self.thirty_days_from_now = (datetime.now() + timedelta(days=30)).strftime('%Y-%m-%d')
        self.fifteen_days_ago = (datetime.now() - timedelta(days=15)).strftime('%Y-%m-%d')
        self.fifteen_days_from_now = (datetime.now() + timedelta(days=15)).strftime('%Y-%m-%d')
        self.eight_days_from_now = (datetime.now() + timedelta(days=8)).strftime('%Y-%m-%d')
        self.eight_days_ago = (datetime.now() - timedelta(days=8)).strftime('%Y-%m-%d')




    async def rsi_scanner(self, ticker):
        rsi = await self.rsi(ticker=ticker, timespan='hour')

        if rsi is not None:
            rsi = rsi.rsi_value[0]
            print(rsi)


scanner = RSIScanner(os.environ.get('POLYGON_STRING'))
async def main():
    results = await scanner.rsi_scanner('AAPL')


asyncio.run(main())






class InstitutionHolding:
    def __init__(self, data):
        self.institution_holding = InstitutionStat(data['institutionHolding'])

    def to_dict(self):
        return {
            'institutionHolding': self.institution_holding.to_dict()
        }

class InstitutionStat:
    def __init__(self, data):
        self.stat = Stat(data['institutionHolding']['stat'])
        self.new_position = Position(data['institutionHolding']['newPosition'])
        self.increase = Position(data['institutionHolding']['increase'])
        self.sold_out = Position(data['institutionHolding']['soldOut'])
        self.decrease = Position(data['institutionHolding']['decrease'])

    def to_dict(self):
        return {
            'stat': self.stat.to_dict(),
            'newPosition': self.new_position.to_dict(),
            'increase': self.increase.to_dict(),
            'soldOut': self.sold_out.to_dict(),
            'decrease': self.decrease.to_dict(),
        }

class Stat:
    def __init__(self, data):
        self.holding_count = data.get('holdingCount', None)
        self.holding_count_change = data.get('holdingCountChange', None)
        self.holding_ratio = data.get('holdingRatio', None)
        self.holding_ratio_change = data.get('holdingRatioChange', None)
        self.institutional_count = data.get('institutionalCount', None)

    def to_dict(self):
        return {
            'holdingCount': self.holding_count,
            'holdingCountChange': self.holding_count_change,
            'holdingRatio': self.holding_ratio,
            'holdingRatioChange': self.holding_ratio_change,
            'institutionalCount': self.institutional_count,
        }

class Position:
    def __init__(self, data):
        self.holding_count_change = data.get('holdingCountChange', None)
        self.institutional_count = data.get('institutionalCount', None)

    def to_dict(self):
        return {
            'holdingCountChange': self.holding_count_change,
            'institutionalCount': self.institutional_count,
        }



import requests
import pandas as pd

def get_survey_series():
    r = requests.get("https://api.bls.gov/publicAPI/v2/surveys").json()
    results = r['Results']
    survey = results['survey']
    df = pd.DataFrame(survey)
    df.to_csv('survey_dict.csv', index=False)
    # Initialize an empty list to store (survey_name, series_id) tuples
    survey_series_list = []

    # Iterate through the survey abbreviations to fetch series IDs
    for index, row in df.iterrows():
        survey_abbreviation = row['survey_abbreviation']
        survey_name = row['survey_name']

        url = f"https://api.bls.gov/publicAPI/v2/timeseries/popular?survey={survey_abbreviation}"
        r = requests.get(url).json()
        try:
            series_ids = [item['seriesID'] for item in r.get('Results', {}).get('series', [])]
            print(series_ids)
            for series_id in series_ids:
                survey_series_list.append((survey_name, series_id))
        except TypeError:
            continue

    df2 = pd.DataFrame(survey_series_list)

    df2.to_csv('all_survey_series.csv', index=False)
    print('dataframe of surveys successfully saved.')


# Read the saved CSV into a DataFrame
df_series_ids = pd.read_csv('all_survey_series.csv')

# Initialize an empty list to store data
data_list = []

# Iterate through the Series IDs to fetch data
for index, row in df_series_ids.iterrows():
    survey_name = row['Survey Name']
    series_id = row['Series ID']

    url = f"https://api.bls.gov/publicAPI/v2/timeseries/data/{series_id}"
    r = requests.get(url).json()

    results = r['Results'] if 'Results' in r else None
    if results is not None:
        series = results['series'] if 'series' in results else None
        if series is not None:
            data = [i['data'] if 'data' in i else None for i in series]

            year = [i[0].get('year') for i in data]
            period = [i[0].get('period') for i in data]
            period_name = [i[0].get('periodName') for i in data]
            value = [i[0].get('value') for i in data]


            data_dict = { 
                'survey': survey_name,
                'series': series_id,
                'year': year[0],
                'period':period[0],
                'period_name': period_name[0],
                'value': value[0]
            }

            data_list.append(data_dict)


    df = pd.DataFrame(data_list)

    df.to_csv('all_economic_data.csv', index=False)




import asyncio
from fudstop.apis.polygonio.polygon_options import PolygonOptions
from fudstop.list_sets.ticker_lists import most_active_tickers

opts = PolygonOptions()

async def main(ticker, semaphore):
    async with semaphore:
        all_options = await opts.find_symbols(ticker)
        df = all_options.df
        print(df)
        await opts.batch_insert_dataframe(df, table_name='poly_options', unique_columns='option_symbol')

async def update_all_options():
    pool = await opts.connect()
    semaphore = asyncio.Semaphore(3)  # Adjust the number as needed
    tasks = [main(i, semaphore) for i in most_active_tickers]
    await asyncio.gather(*tasks)

asyncio.run(update_all_options())